/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <stdio.h>

#include "read_icon_data.h"
#include "read_fortran_data.h"

/* --------------------------------------------------- */

/* Fortran file functions for reading header and trailer of records */

static int header ( FILE *fileId );

static void trailer ( FILE *fileId );

/* --------------------------------------------------- */

void read_fortran_dimensions(
  FILE *fileId, struct fieldMetadata * fieldInfo, int num_cells) {

  int size;

  int field_id;
  int year, month, day, hour, minute;
  double second;

  /* File structure of coupling restarts */

  /* 1st) field id */

  size = header(fileId);
  fread(&field_id, sizeof(int), 1, fileId);
  trailer(fileId);

  /* 2nd) time stamp */

  size = header(fileId);
  fread(&year,   sizeof(int), 1, fileId);
  fread(&month,  sizeof(int), 1, fileId);
  fread(&day,    sizeof(int), 1, fileId);
  fread(&hour,   sizeof(int), 1, fileId);
  fread(&minute, sizeof(int), 1, fileId);
  fread(&second, sizeof(double), 1, fileId);
  trailer(fileId);

  /* 3rd) accumulation count */

  size = header(fileId);
  fread(&fieldInfo->accumulatedSteps, sizeof(int), 1, fileId);
  trailer(fileId);

  /* 4th) data size */

  size = header(fileId);
  fread(&fieldInfo->nbrHorPoints, sizeof(int), 1, fileId);
  fread(&fieldInfo->nbrBundles, sizeof(int), 1, fileId);
  trailer(fileId);

  printf ( "field ID %i \n", field_id );
  printf ( "Date: %i %i %i T %i : %i : %f \n", 
           year, month, day, hour, minute, second );
  printf ( "number of accumulated steps: %i \n", fieldInfo->accumulatedSteps);
  printf ( "sizes %i %i \n", fieldInfo->nbrHorPoints, fieldInfo->nbrBundles );

  /* 5th) data section */

  size = header(fileId);

  if ( fieldInfo->nbrHorPoints * fieldInfo->nbrBundles != size / (int)sizeof(double) ) {
    printf("The expected size does not match with the record length! %i %i \n",
           fieldInfo->nbrHorPoints, size / (int) sizeof(double) );
    exit(1);
  }

  if (  fieldInfo->nbrHorPoints != num_cells ) {
    printf("Wrong grid file? Number of cells does not match! %i %i \n", 
           fieldInfo->nbrHorPoints, num_cells  );
    exit(1);
  }

}

/* --------------------------------------------------- */

void read_fortran_data(
  double * cell_data, struct fieldMetadata fieldInfo, FILE *fileId) {
  fread(cell_data, sizeof(double), fieldInfo.nbrHorPoints * fieldInfo.nbrBundles, fileId);
  trailer(fileId);
}

/* --------------------------------------------------- */

static int header ( FILE *fileId ) {
  int head;
  fread(&head, sizeof(int), 1, fileId);
#ifdef DEBUG
  printf("header: %i bytes \n", head);
#endif
  return head;
}

static void trailer ( FILE *fileId ) {
  int tail;
  fread(&tail, sizeof(int), 1, fileId);
#ifdef DEBUG
  printf("trailer: %i bytes \n", tail);
#endif
}

