/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef FORTRAN_DATA_H
#define FORTRAN_DATA_H

#include <stdlib.h>

/** \file read_fortran_data.h
  * \brief general routines for reading Fortran binary file from c
  *
  * These routines should be called in a specific order:
  *
  * -# \ref read_fortran_dimensions
  * -# \ref read_fortran_data
  *
  * \remark These routines are adapted to read the current ICON coupler restart files
  * \remark and are not meant to be general purpose routines. 
 **/

/**
 * reads some metadata from the Fortran file
 * @param[in]  fileId    file pointer to an already open file
 * @param[in]  numCells  expected size of the data
 * @param[out] fieldInfo metadata information about the stored array which has been written into the Fortran file
 */
void read_fortran_dimensions ( FILE *fileId, struct fieldMetadata * fieldInfo, int numCells );

/**
 * reads (Fortran DOUBLE PRECISION) data from the Fortran file
 * @param[out] cell_data guess what?
 * @param[in]  fieldInfo metadata information about the stored array which has been written into the Fortran file
 * @param[out] fileId    file pointer to an already open file
 */
void read_fortran_data ( double * cell_data, struct fieldMetadata fieldInfo, FILE *fileId);

#endif

