package de.mpg.mpimet.yacgui.ui.dialog.tabpanel;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import de.mpg.mpimet.yacgui.xml.DebugMode;
import de.mpg.mpimet.yacgui.xml.TransientCouple;

/**
 * Copyright (c) 2013 Max-Planck-Institute for Meteorology, Hamburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

public class MoreTab extends AbstractTab {

	private static final long serialVersionUID = -4615817689584953223L;

	private static final String LABEL_DEBUG_MODE = "Debug mode: ";
	private static final String LABEL_ENFORCE_WRITE_RESTART = "Enforce write restart";
	private static final String LABEL_MASK_VALUE = "Mask value (optional): ";
	
	private static final String CHECKBOX_BEFORE_INTERPOLATION = " at source before interpolation";
	private static final String CHECKBOX_AFTER_INTERPOLATION = " at source after interpolation";
	private static final String CHECKBOX_AT_TARGET = " at target";
	
	private static final int DEFAULT_COLUMN_NUM = 7;

	private Double maskValue;
	private DebugMode debugMode;
	private boolean enforceWriteRestart;
	
	private JCheckBox atSourceBeforeInterpolationCheckBox;
	private JCheckBox atSourceAfterInterpolationCheckBox;
	private JCheckBox atTargetCheckBox;

	private JCheckBox enforceWriteRestartCheckBox;

	private JCheckBox maskValueCheckBox;
	private JFormattedTextField maskValueTextField;

	public MoreTab(String dialogDescription, TransientCouple transientCouple) {
		super(dialogDescription, transientCouple);
		createLayout();
	}

	public void collectData() {
		debugMode.setAtSourceBeforeInterpolation(atSourceBeforeInterpolationCheckBox.isSelected());
		debugMode.setAtSourceAfterInterpolation(atSourceAfterInterpolationCheckBox.isSelected());
		debugMode.setAtTarget(atTargetCheckBox.isSelected());
		
		transientCouple.setEnforceWriteRestart(enforceWriteRestartCheckBox.isSelected());
		
		if (maskValueCheckBox.isSelected()) {
			maskValue = ((Number) maskValueTextField.getValue()).doubleValue();
			transientCouple.setMaskValue(maskValue);
		} else {
			transientCouple.setMaskValue(null);
		}
	}

	@Override
	protected JPanel createCenterPanel() {
		JPanel moreOptionsPanel = new JPanel();
		moreOptionsPanel.setLayout(new BoxLayout(moreOptionsPanel, BoxLayout.PAGE_AXIS));
		
		// debug mode
		JPanel debugModePanel = new JPanel();
		debugModePanel.setLayout(new BoxLayout(debugModePanel, BoxLayout.PAGE_AXIS));
		JLabel debugModeLabel = new JLabel(LABEL_DEBUG_MODE);
		debugModeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModeLabel.setBorder(new EmptyBorder(5, 5, 5, 5));
		debugModePanel.add(debugModeLabel);
		
		// at source before interpolation
		atSourceBeforeInterpolationCheckBox = new JCheckBox(CHECKBOX_BEFORE_INTERPOLATION);
		atSourceBeforeInterpolationCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModePanel.add(atSourceBeforeInterpolationCheckBox);
		
		// at source after interpolation
		atSourceAfterInterpolationCheckBox = new JCheckBox(CHECKBOX_AFTER_INTERPOLATION);
		atSourceAfterInterpolationCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModePanel.add(atSourceAfterInterpolationCheckBox);
		
		// at target
		atTargetCheckBox = new JCheckBox(CHECKBOX_AT_TARGET);
		atTargetCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		debugModePanel.add(atTargetCheckBox);

		// enforce write-restart
		JPanel enforceWriteRestartPanel = new JPanel();
		enforceWriteRestartPanel.setLayout(new BoxLayout(enforceWriteRestartPanel, BoxLayout.PAGE_AXIS));
		enforceWriteRestartCheckBox = new JCheckBox(LABEL_ENFORCE_WRITE_RESTART);
		enforceWriteRestartCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		enforceWriteRestartPanel.add(enforceWriteRestartCheckBox);
		
		// mask value checkbox
		JPanel maskPanel = new JPanel();
		maskValueCheckBox = new JCheckBox(LABEL_MASK_VALUE);
		maskPanel.add(maskValueCheckBox);
		
		// mask value input
		NumberFormat format = NumberFormat.getNumberInstance();
		format.setMinimumFractionDigits(1);
		maskValueTextField = new JFormattedTextField(format);
		maskValueTextField.setColumns(DEFAULT_COLUMN_NUM);
		maskValueTextField.setHorizontalAlignment(JTextField.RIGHT);
		
		maskPanel.add(maskValueTextField);
		
		// mask value checkbox action listener
		maskValueCheckBox.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				maskValueTextField.setEnabled(maskValueCheckBox.isSelected());
			}
		});
		
		moreOptionsPanel.add(debugModePanel);
		moreOptionsPanel.add(Box.createVerticalGlue());
		moreOptionsPanel.add(enforceWriteRestartPanel);
		moreOptionsPanel.add(Box.createVerticalGlue());
		moreOptionsPanel.add(maskPanel);
		
		loadParameters();
		
		return moreOptionsPanel;
	}
	
	@Override
	public void loadParameters() {
		
		this.maskValue = transientCouple.getMaskValue();
		this.debugMode = transientCouple.getDebugMode();
		this.enforceWriteRestart = transientCouple.isEnforceWriteRestart();
		
		atSourceBeforeInterpolationCheckBox.setSelected(debugMode.isAtSourceBeforeInterpolation());
		atSourceAfterInterpolationCheckBox.setSelected(debugMode.isAtSourceAfterInterpolation());
		atTargetCheckBox.setSelected(debugMode.isAtTarget());
		enforceWriteRestartCheckBox.setSelected(enforceWriteRestart);
		
		maskValueCheckBox.setSelected(maskValue != null);
		
		if (maskValue != null) {
			maskValueTextField.setValue(Double.valueOf(maskValue));
			maskValueTextField.setEnabled(true);
		} else {
			maskValueTextField.setValue(Double.valueOf(0.0));
			maskValueTextField.setEnabled(false);
		}		
	}
}
