package de.mpg.mpimet.yacgui.xml;

import java.util.Map;

/**
 * Copyright (c) 2013 Max-Planck-Institute for Meteorology, Hamburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Java representation of {@code <component>} xml element.
 * 
 * @author Maxim Yastremsky
 */

public class Component implements Comparable<Component> {
	
	private int id;
	private String name;
	private String model;
	private String simulated;
	
	/** TransientGridRef.id : TransientGridRef */
	private Map<Integer, TransientGridRef> transientGridRefs;
	
	/** Transient.id : Transient. Only for component.xsd */
	private Map<Integer, Transient> transients;
	
	/** Grid.id : Grid. Only for component.xsd */
	private Map<Integer, Grid> grids;
	
	public Component(int id, String name, String model, String simulated, Map<Integer, TransientGridRef> transientGridRefs) {
		this.id = id;
		this.name = name;
		this.model = model;
		this.simulated = simulated;
		this.transientGridRefs = transientGridRefs;
	}

	public int getId() {
		return id;
	}

	public String getName() {
		return name;
	}

	public String getModel() {
		return model;
	}

	public String getSimulated() {
		return simulated;
	}

	public Map<Integer, TransientGridRef> getTransientGridRefs() {
		return transientGridRefs;
	}

	public Map<Integer, Transient> getTransients() {
		return transients;
	}

	public Map<Integer, Grid> getGrids() {
		return grids;
	}
	
	public void setTransients(Map<Integer, Transient> transients) {
		this.transients = transients;
	}
	
	public void setGrids(Map<Integer, Grid> grids) {
		this.grids = grids;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((grids == null) ? 0 : grids.hashCode());
		result = prime * result + id;
		result = prime * result + ((model == null) ? 0 : model.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((simulated == null) ? 0 : simulated.hashCode());
		result = prime * result + ((transientGridRefs == null) ? 0 : transientGridRefs.hashCode());
		result = prime * result + ((transients == null) ? 0 : transients.hashCode());
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof Component)) {
			return false;
		}
		Component other = (Component) obj;
		if (grids == null) {
			if (other.grids != null) {
				return false;
			}
		} else if (!grids.equals(other.grids)) {
			return false;
		}
		if (id != other.id) {
			return false;
		}
		if (model == null) {
			if (other.model != null) {
				return false;
			}
		} else if (!model.equals(other.model)) {
			return false;
		}
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (simulated == null) {
			if (other.simulated != null) {
				return false;
			}
		} else if (!simulated.equals(other.simulated)) {
			return false;
		}
		if (transientGridRefs == null) {
			if (other.transientGridRefs != null) {
				return false;
			}
		} else if (!transientGridRefs.equals(other.transientGridRefs)) {
			return false;
		}
		if (transients == null) {
			if (other.transients != null) {
				return false;
			}
		} else if (!transients.equals(other.transients)) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(Component o) {
		return id - o.id;
	}
}
