package de.mpg.mpimet.yacgui.xml.interpolation;

import de.mpg.mpimet.yacgui.xml.Dom;

/**
 * Copyright (c) 2013 Max-Planck-Institute for Meteorology, Hamburg.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * Definite interpolation method patch recovery
 * 
 * @author Maxim Yastremsky
 */

public class PatchRecoveryInterpolation extends Interpolation {

	private boolean allowExtrapolation;
	private int extendSourcePatch;
	private int polynomialOrder;
	private int gaussOrder;
	
	public PatchRecoveryInterpolation(boolean allowExtrapolation, int extendSourcePatch,
			int polynomialOrder, int gaussOrder) throws IllegalArgumentException {
		
		if (extendSourcePatch < 1) {
			throw new IllegalArgumentException("ExtendSourcePatch must be bigger than 0");
		}
		if (polynomialOrder < 1) {
			throw new IllegalArgumentException("PolynomialOrder must be bigger than 0");
		}
		if (gaussOrder < 1) {
			throw new IllegalArgumentException("GaussOrder must be bigger than 0");
		}
		
		attributesNumber = 4;
		method = InterpolationMethod.PATCH_RECOVERY;
		this.allowExtrapolation = allowExtrapolation;
		this.extendSourcePatch = extendSourcePatch;
		this.polynomialOrder = polynomialOrder;
		this.gaussOrder = gaussOrder;
	}
	
	/**
	 * 
	 * @return true if extrapolation is allowed in patch recovery  
	 */
	public boolean isAllowExtrapolation() {
		return allowExtrapolation;
	}

	/**
	 * 
	 * @return integer value of extend source patch
	 */
	public int getExtendSourcePatch() {
		return extendSourcePatch;
	}
	
	/**
	 * 
	 * @return integer value of polynomial order
	 */
	public int getPolynomialOrder() {
		return polynomialOrder;
	}
	

	/**
	 * 
	 * @return integer value of gauss order
	 */
	public int getGaussOrder() {
		return gaussOrder;
	}
	
	/**
	 * get attribute name by the order of index
	 * @param index the order of attributes on the panel
	 * @return  the name of attribute
	 * 				if index = 0 return ALLOW_EXTRAPOLATION, if 1 return EXTEND_SOURCE_PATCH, if 2 return POLYNOMIAL_ORDER, 
	 * if 3 return GAUSS_ORDER
	 */
	public String getAttributeName(int index) {
		if (index == 0) {
			return Dom.Attribute.ALLOW_EXTRAPOLATION;
		} else if (index == 1) {
			return Dom.Attribute.EXTEND_SOURCE_PATCH;
		} else if (index == 2) {
			return Dom.Attribute.POLYNOMIAL_ORDER;
		} else if (index == 3) {
			return Dom.Attribute.GAUSS_ORDER;
		}
		return null;
	}
	
	/**
	 * get attribute value by the order of index
	 * @param index the order of attributes on the panel
	 * @return the value of attribute as String
	 *				if 0 return allowExtrapolation, if 1 return extendSourcePatch, if 2 return polynomialOrder, 
	 * if 3 return gaussOrder
	 */
	public String getAttributeValue(int index) {
		if (index == 0) {
			return String.valueOf(allowExtrapolation);
		} else if (index == 1) {
			return String.valueOf(extendSourcePatch);
		} else if (index == 2) {
			return String.valueOf(polynomialOrder);
		} else if (index == 3) {
			return String.valueOf(gaussOrder);
		}
		return null;
	}
}
