/**
 * @file field_interpolation_gen_tgt.h
 *
 * @copyright Copyright  (C)  2013 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef FIELD_INTERPOLATION_GEN_TGT_H
#define FIELD_INTERPOLATION_GEN_TGT_H

enum yac_field_interpolation_tgt_mask_handling {

   TGT_IGNORE_MASK, //!< ignores the mask (this is the default)
   TGT_NO_INTERP,   //!< masked target points will no be
                    //!< interpolated
};

struct field_interpolation_tags;

/**
 * constructor for a field_interpolation_gen on the target side
 * @param[in] points                  point sets
 * @param[in] num_pointsets           number of point sets in points
 * @param[in] field_config_name       name if the target field configuration
 * @param[in] search                  global search used as the basis for the
 *                                    interpolation
 * @param[in] other_field_config_name name of the source field configuration
 * @param[in] points_idx              which points set is to of the domain is to
 *                                    interpolated
 * @param[in] mapping_on_source       is the mapping in the put-get-operation to
 *                                    be executed on the source processes (on
 *                                    the target otherwise)
 * @return field interpolation generator
 */
struct field_interpolation_gen *
yac_new_field_interpolation_gen_tgt(struct points *** points,
                                    unsigned num_pointsets,
                                    char const * field_config_name,
                                    struct global_search * search,
                                    const char * other_field_config_name,
                                    unsigned points_idx, int mapping_on_source);

struct field_interpolation_gen *
yac_new_field_interpolation_gen_tgt_(struct points *** points,
                                     unsigned num_pointsets,
                                     char const * field_config_name,
                                     struct global_search * search,
                                     const char * other_field_config_name,
                                     unsigned points_idx,
                                     struct field_interpolation_tags tags,
                                     int mapping_on_source);

/**
 * constructor for a field_interpolation_gen on the target side
 * @param[in] points                  point sets
 * @param[in] masks                   masks
 * @param[in] num_pointsets           number of point sets in points
 * @param[in] field_config_name       name if the target field configuration
 * @param[in] search                  global search used as the basis for the
 *                                    interpolation
 * @param[in] other_field_config_name name of the source field configuration
 * @param[in] points_idx              which points set is to of the domain is to
 *                                    interpolated
 * @param[in] mask_handling           how to handle masked target points
 * @param[in] mapping_on_source       is the mapping in the put-get-operation to
 *                                    be executed on the source processes (on
 *                                    the target otherwise)
 * @return field interpolation generator
 */
struct field_interpolation_gen *
yac_new_field_interpolation_gen_tgt_mask(struct points *** points,
                                         struct mask *** masks,
                                         unsigned num_pointsets,
                                         char const * field_config_name,
                                         struct global_search * search,
                                         const char * other_field_config_name,
                                         unsigned points_idx,
                                         enum yac_field_interpolation_tgt_mask_handling
                                          mask_handling, int mapping_on_source);

struct field_interpolation_gen *
yac_new_field_interpolation_gen_tgt_mask_(struct points *** points,
                                          struct mask *** masks,
                                          unsigned num_pointsets,
                                          char const * field_config_name,
                                          struct global_search * search,
                                          const char * other_field_config_name,
                                          unsigned points_idx,
                                          enum yac_field_interpolation_tgt_mask_handling
                                             mask_handling,
                                          struct field_interpolation_tags tags,
                                          int mapping_on_source);

#endif // FIELD_INTERPOLATION_GEN_TGT_H

