/**
 * @file grid_reg2d_common.h
 * @brief routines shared between grid_reg2d and grid_curve2d
 *
 * @copyright Copyright  (C)  2017 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GRID_REG2D_COMMON_H
#define GRID_REG2D_COMMON_H

#include "grid.h"

int yac_get_aux_grid_cell_reg2d_common(
  unsigned num_cells[2], unsigned cyclic, unsigned corner_index,
  unsigned * cell_indices);

unsigned yac_get_num_corner_edges_reg2d_common(
  unsigned num_cells[2], unsigned cyclic, unsigned corner_index);

struct dep_list * yac_generate_cell_neigh_dep_reg2d_common(
  unsigned num_cells[2], unsigned cyclic[2]);

void yac_get_boundary_corners_reg2d_common(
  unsigned num_cells[2], unsigned cyclic, unsigned * bnd_corners,
  unsigned * num_bnd_corners);

unsigned const * yac_get_corner_edges_reg2d_common(
  unsigned num_cells[2], unsigned cyclic, unsigned corner_index);

unsigned const * yac_get_corner_edge_indices_reg2d_common_cyclic(
  unsigned num_cells[2], unsigned row, unsigned column);

unsigned const * yac_get_corner_edge_indices_reg2d_common_non_cyclic(
  unsigned num_cells[2], unsigned row, unsigned column);

unsigned const * yac_get_cell_edge_indices_reg2d_common_cyclic(
   unsigned num_cells[2], unsigned row, unsigned column);

unsigned const * yac_get_cell_edge_indices_reg2d_common_non_cyclic(
   unsigned num_cells[2], unsigned row, unsigned column);

unsigned const * yac_get_cell_corner_indices_reg2d_common(
  unsigned num_cells[2], unsigned cyclic, unsigned cell_index);

unsigned const * yac_get_corner_cell_indices_reg2d_common(
  unsigned num_cells[2], unsigned cyclic, unsigned corner_index);

struct grid * yac_generate_edge_grid_reg2d_common(
  unsigned num_cells[2], unsigned cyclic[2],
  double * coordinates_x, double * coordinates_y);

#endif // GRID_REG2D_COMMON_H

