/**
 * @file grid_unpack.c
 * @brief Generic rountine to initialise packing of grids.
 *
 * @copyright Copyright  (C)  2013 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>

#include "grid.h"
#include "grid_reg2d.h"
#include "grid_curve2d.h"
#include "grid_unstruct.h"
#include "utils.h"
#include "communicator.h"

struct grid * yac_unpack_grid(void * buffer, int buffer_size, int * position,
                              struct communicator * comm) {

  unsigned grid_type;
  yac_comm_unpack(
    buffer, buffer_size, position, &grid_type, 1, COMM_UINT, comm);

  switch ((enum yac_grid_type)grid_type) {
    case (YAC_REG2D):
     return yac_unpack_reg2d_grid(buffer, buffer_size, position, comm);
    case (YAC_CURVE2D):
     return yac_unpack_curve2d_grid(buffer, buffer_size, position, comm);
    case (YAC_UNSTRUCT):
     return yac_unpack_unstruct_grid(buffer, buffer_size, position, comm);
    default:
     yac_internal_abort_message(
        "ERROR: grid type must be one of YAC_REG2D|YAC_CURVE2D|YAC_UNSTRUCT.",
        __FILE__, __LINE__);
    return NULL;
  }
}

