/**
 * @file interpolation_method_file.h
 * @brief weight file based interpolation method
 *
 * point interpolation method that reads weights from a file in SCRIP format
 *
 * @copyright Copyright  (C)  2015 Moritz Hanke <hanke@dkrz.de>
 *                                 Rene Redler <rene.redler@mpimet.mpg.de>
 *
 * @version 1.0
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INTERPOLATION_METHOD_FILE_H
#define INTERPOLATION_METHOD_FILE_H

#include "points.h"
#include "interpolation_method.h"

#define YAC_WEIGHT_FILE_VERSION_STRING "yac weight file 1.0"

/** \example test_interpolation_method_file.c
* This shows how the file interpolation works
*/
 
/**
 * constructor for a interpolation method of type interpolation_method_file
 * @param[in] search        global search object that is to be used for the
 *                          interpolation
 * @param[in] domain        %domain of the local process
 * @param[in] stack_name    name of the interpolation stack
 * @param[in] src_locations location for each pointset
 * @param[in] num_pointsets number of point sets of the field
 * @param[in] file_name     name of file that contains the link and weights
 *                          information
 * @param[in] io_proc_step  stride between ranks of io processes
 */
struct interpolation_method *
yac_new_interpolation_method_file(struct global_search * search,
                                  struct domain * domain,
                                  char const * stack_name,
                                  enum yac_location * src_locations,
                                  unsigned num_pointsets,
                                  char const * file_name,
                                  unsigned io_proc_step);

#endif // INTERPOLATION_METHOD_FILE_H

