/**
 * @file interpolation_enum.h
 *
 * @copyright Copyright  (C)  2013 Rene Redler <rene.redler@mpimet.mpg.de>
 *                                 Teresa Holfeld <teresa.holfeld@zmaw.de>
 *
 * @version 1.0
 * @author Rene Redler <rene.redler@mpimet.mpg.de>
 *         Teresa Holfeld <teresa.holfeld@zmaw.de>
 */
/*
 * Keywords:
 * Maintainer: Rene Redler <rene.redler@mpimet.mpg.de>
 *             Teresa Holfeld <teresa.holfeld@zmaw.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef INTERPOLATION_ENUM_H_
#define INTERPOLATION_ENUM_H_

enum yac_interpolation_list {
   UNDEFINED             =  0,
   AVERAGE               =  1,
   N_NEAREST_NEIGHBOR    =  2,
   CONSERVATIVE          =  3,
   PATCH_RECOVERY        =  4,
   SMOOTH_PATCH_RECOVERY =  5,
   SOURCE_TO_TARGET_MAP  =  6,
   FIXED_VALUE           =  7,
   USER_FILE             =  8,
   CHECK                 =  9,
   BERNSTEIN_BEZIER      = 10,
   RADIAL_BASIS_FUNCTION = 11,
};

char * yac_interpolation_enum_to_string(const enum yac_interpolation_list interpolation);

char * yac_interpolation_int_to_string(const int interpolation);

enum yac_interpolation_list yac_interpolation_enum_from_string(const char * interpolation);

/**
 * Verifies if interpolation string matches one of the given interpolation
 * methods in enum.
 *
 * @return the interpolation string, or "user_defined" if string doesn't match
 */
char * yac_verified_interpolation(const char * interpolation);

#endif /* INTERPOLATION_ENUM_H_ */

