/**
 * @file test_bisection.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <stdio.h>
#include "utils.h"
#include "tests.h"
#include "search.h"

int main (void) {

#define AXIS_LENGTH (511)
#define DATA_LENGTH (8)
#define TEST_LENGTH (100)
#define NBR_TESTS (10)

   double axis_data[AXIS_LENGTH];
   double search_data[DATA_LENGTH];
   double test_data[TEST_LENGTH];

   double frac;

   int * position, * found;

   unsigned i;

   position = xmalloc(MAX(DATA_LENGTH, TEST_LENGTH) * sizeof(*position));
   found = xmalloc(MAX(DATA_LENGTH, TEST_LENGTH) * sizeof(*found));

   // set the test data
   search_data[0] = -0.5;
   search_data[1] =  0.0;
   search_data[2] =  0.5;
   search_data[3] = 40;
   search_data[4] = 250.5;
   search_data[5] = (double)(AXIS_LENGTH-1) - 0.5;
   search_data[6] = (double)(AXIS_LENGTH-1);
   search_data[7] = (double)(AXIS_LENGTH-1) + 0.5;

#ifdef VERBOSE
   puts ("----- ascending axis data -----");
#endif
   
   // generate the axis data
   for (i = 0; i < AXIS_LENGTH; ++i) {
      axis_data[i] = (double)i;
   }

   yac_bisection_search(search_data, DATA_LENGTH, axis_data, AXIS_LENGTH, position,
                        found, 0);

#ifdef VERBOSE
   for (i = 0; i < DATA_LENGTH; ++i) {
      if (found[i])
         printf("%6.1f in box %3d : %6.1f - %6.1f %c\n", search_data[i], position[i], axis_data[position[i]], axis_data[position[i]+1], found[i]?'T':'F');
      else
         printf("%6.1f                              F\n", search_data[i]);
   }

   puts ("----- descending axis data -----");
#endif

   // generate the axis data
   for (i = 0; i < AXIS_LENGTH; ++i) {

      axis_data[i] = (double)(AXIS_LENGTH-1-i);
   }

   yac_bisection_search(search_data, DATA_LENGTH, axis_data, AXIS_LENGTH, position,
                        found, 0);

#ifdef VERBOSE
   for (i = 0; i < DATA_LENGTH; ++i) {
      if (found[i])
         printf("%6.1f in box %3d : %6.1f - %6.1f %c\n", search_data[i], position[i], axis_data[position[i]], axis_data[position[i]+1], found[i]?'T':'F');
      else
         printf("%6.1f                              F\n", search_data[i]);
   }
#endif

   // do some longer test for performance analysis

   frac = (double)(AXIS_LENGTH) / (double)(TEST_LENGTH);

   // re-generate the axis data
   for (i = 0; i < AXIS_LENGTH; ++i) {
      axis_data[i] = (double)i;
   }
   // generate the test data
   for (i=0; i < TEST_LENGTH;  ++i) {
     test_data[i] = (double)(i+1) * frac;
   }

   for (i = 0; i < NBR_TESTS; ++i) {
      yac_bisection_search(test_data, TEST_LENGTH, axis_data, AXIS_LENGTH, position,
                           found, 0);
   }

   free(position);
   free(found);

   return TEST_EXIT_CODE;
}

