#define VERBOSE

/**
 * @file test_check_overlap.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "tests.h"
#include "search.h"
#include "test_common.h"

int main (void) {

   {
      //---------------
      // setup
      //---------------
      struct grid_cell A, B;
      double A_x [] = {-16.9189067783954670,
                       -14.8983498405395949,
                       -14.8663077006549429,
                       -16.8935456207927928};
      double A_y [] = {42.6693139217827593,
                       42.6505696135177459,
                       44.1537492302363717,
                       44.1757425575779052};
      double B_x [] = {-15.6117753982543945,
                       -15.0301752090454102,
                       -15.2238006591796875,
                       -15.8087253570556641};
      double B_y [] = {42.9113769531250000,
                       43.0533523559570312,
                       43.4789505004882812,
                       43.3361511230468750};

      for ( unsigned i = 0; i < 4; ++i ) {
         A_x [i] *= YAC_RAD;
         A_y [i] *= YAC_RAD;
         B_x [i] *= YAC_RAD;
         B_y [i] *= YAC_RAD;
      }

      enum yac_edge_type edge_type[4] = {GREAT_CIRCLE, GREAT_CIRCLE,
                                         GREAT_CIRCLE, GREAT_CIRCLE};

      A.coordinates_x = A_x;
      A.coordinates_y = A_y;
      A.coordinates_xyz = (double[4][3]){{-1}};
      A.num_corners = 4;
      A.edge_type = edge_type;
      B.coordinates_x = B_x;
      B.coordinates_y = B_y;
      B.coordinates_xyz = (double[4][3]){{-1}};
      B.num_corners = 4;
      B.edge_type = edge_type;

      generate_XYZ(A);
      generate_XYZ(B);

      //---------------
      // testing
      if (!yac_check_overlap_cells(A,A)) PUT_ERR("error in check_overlap_cells(A,A) 1\n")
      if (!yac_check_overlap_cells(B,B)) PUT_ERR("error in check_overlap_cells(B,B) 1\n")
      if (!yac_check_overlap_cells(A,B)) PUT_ERR("error in check_overlap_cells(A,B) 1\n")
      if (!yac_check_overlap_cells(B,A)) PUT_ERR("error in check_overlap_cells(B,A) 1\n")
   }

   {
      //---------------
      // setup
      //---------------
      struct grid_cell A, B;
      double A_x [] = {15, 165, 120, 70};
      double A_y [] = {89.5, 89.5, 89, 89};
      double B_x [] = {240, 290, 345, 195};
      double B_y [] = {89, 89, 89.5, 89.5};

      for ( unsigned i = 0; i < 4; ++i ) {
         A_x [i] *= YAC_RAD;
         A_y [i] *= YAC_RAD;
         B_x [i] *= YAC_RAD;
         B_y [i] *= YAC_RAD;
      }

      enum yac_edge_type edge_type[4] = {GREAT_CIRCLE, GREAT_CIRCLE,
                                         GREAT_CIRCLE, GREAT_CIRCLE};

      A.coordinates_x = A_x;
      A.coordinates_y = A_y;
      A.coordinates_xyz = (double[4][3]){{-1}};
      A.num_corners = 4;
      A.edge_type = edge_type;
      B.coordinates_x = B_x;
      B.coordinates_y = B_y;
      B.coordinates_xyz = (double[4][3]){{-1}};
      B.num_corners = 4;
      B.edge_type = edge_type;

      generate_XYZ(A);
      generate_XYZ(B);

      //---------------
      // testing
      //---------------
      if (!yac_check_overlap_cells(A,A)) PUT_ERR("error in check_overlap_cells(A,A) 2\n")
      if (!yac_check_overlap_cells(B,B)) PUT_ERR("error in check_overlap_cells(B,B) 2\n")
      if (yac_check_overlap_cells(A,B))  PUT_ERR("error in check_overlap_cells(A,B) 2\n")
      if (yac_check_overlap_cells(B,A))  PUT_ERR("error in check_overlap_cells(B,A) 2\n")
   }

   { // checks a very mean case: these cells only overlap if your use greate circles for
     // connecting the points of cell A (this is not done in the old check_overlap_cells)
      //---------------
      // setup
      //---------------
      struct grid_cell A, B;
      double A_x [] = {240, 290, 345, 195};
      double A_y [] = {89, 89, 89.5, 89.5};
      double B_x [] = {270, 150, 90, 30};
      double B_y [] = {89.75, 89.5, 89.25, 89.5};

      for ( unsigned i = 0; i < 4; ++i ) {
         A_x [i] *= YAC_RAD;
         A_y [i] *= YAC_RAD;
         B_x [i] *= YAC_RAD;
         B_y [i] *= YAC_RAD;
      }

      enum yac_edge_type edge_type[4] = {GREAT_CIRCLE, GREAT_CIRCLE,
                                         GREAT_CIRCLE, GREAT_CIRCLE};

      A.coordinates_x = A_x;
      A.coordinates_y = A_y;
      A.coordinates_xyz = (double[4][3]){{-1}};
      A.num_corners = 4;
      A.edge_type = edge_type;
      B.coordinates_x = B_x;
      B.coordinates_y = B_y;
      B.coordinates_xyz = (double[4][3]){{-1}};
      B.num_corners = 4;
      B.edge_type = edge_type;

      generate_XYZ(A);
      generate_XYZ(B);

      //---------------
      // testing
      //---------------
      if (!yac_check_overlap_cells(A,A)) PUT_ERR("error in check_overlap_cells(A,A) 3\n")
      if (!yac_check_overlap_cells(B,B)) PUT_ERR("error in check_overlap_cells(B,B) 3\n")
      if (!yac_check_overlap_cells(A,B)) PUT_ERR("error in check_overlap_cells(A,B) 3\n")
      if (!yac_check_overlap_cells(B,A)) PUT_ERR("error in check_overlap_cells(B,A) 3\n")
   }

   { // check a cased that failed in an icon-gauss example

      //---------------
      // setup
      //---------------
      struct grid_cell A = (struct grid_cell) {
         .coordinates_x = (double[]){-1.7536974358364053e-14,
                                     -1.2909868604205013,
                                     1.290986860420493},
         .coordinates_y = (double[]){29.985135606599414,
                                     28.126432046539517,
                                     28.126432046539499},
         .coordinates_xyz = (double[3][3]){{-1}},
         .edge_type = (enum yac_edge_type[]){GREAT_CIRCLE, GREAT_CIRCLE,
                                             GREAT_CIRCLE},
         .num_corners = 3};

      struct grid_cell B = (struct grid_cell) {
         .coordinates_x = (double[]){0, 1.40625, 1.40625, 0},
         .coordinates_y = (double[]){29.53125, 29.53125, 30.9375, 30.9375},
         .coordinates_xyz = (double[4][3]){{-1}},
         .edge_type = (enum yac_edge_type[]){LAT_CIRCLE, LON_CIRCLE,
                                             LAT_CIRCLE, LON_CIRCLE},
         .num_corners = 4};

      for (size_t i = 0; i < A.num_corners; ++i ) {
         A.coordinates_x[i] *= YAC_RAD;
         A.coordinates_y[i] *= YAC_RAD;
      }
      generate_XYZ(A);

      for (size_t i = 0; i < B.num_corners; ++i ) {
         B.coordinates_x[i] *= YAC_RAD;
         B.coordinates_y[i] *= YAC_RAD;
      }
      generate_XYZ(B);

      //---------------
      // testing
      //---------------
      if (!yac_check_overlap_cells(A,A)) PUT_ERR("error in check_overlap_cells(A,A) 4\n")
      if (!yac_check_overlap_cells(B,B)) PUT_ERR("error in check_overlap_cells(B,B) 4\n")
      if (!yac_check_overlap_cells(A,B)) PUT_ERR("error in check_overlap_cells(A,B) 4\n")
      if (!yac_check_overlap_cells(B,A)) PUT_ERR("error in check_overlap_cells(B,A) 4\n")
   }

   { // checks a cyclic problem case
      //---------------
      // setup
      //---------------
      struct grid_cell A, B;
      double A_x [] = {-5, 5, 5, -5};
      double A_y [] = {-5, -5, 5, 5};
      double B_x [] = {350, 10, 10, 350}; // 10, -10, -10, 10
      double B_y [] = {-10, -10, 10, 10};

      for ( unsigned i = 0; i < 4; ++i ) {
         A_x [i] *= YAC_RAD;
         A_y [i] *= YAC_RAD;
         B_x [i] *= YAC_RAD;
         B_y [i] *= YAC_RAD;
      }

      enum yac_edge_type edge_type[4] = {GREAT_CIRCLE, GREAT_CIRCLE,
                                         GREAT_CIRCLE, GREAT_CIRCLE};

      A.coordinates_x = A_x;
      A.coordinates_y = A_y;
      A.coordinates_xyz = (double[4][3]){{-1}};
      A.num_corners = 4;
      A.edge_type = edge_type;
      B.coordinates_x = B_x;
      B.coordinates_y = B_y;
      B.coordinates_xyz = (double[4][3]){{-1}};
      B.num_corners = 4;
      B.edge_type = edge_type;

      generate_XYZ(A);
      generate_XYZ(B);

      //---------------
      // testing
      //---------------
      if (!yac_check_overlap_cells(A,A)) PUT_ERR("error in check_overlap_cells(A,A) 5\n")
      if (!yac_check_overlap_cells(B,B)) PUT_ERR("error in check_overlap_cells(B,B) 5\n")
      if (!yac_check_overlap_cells(A,B)) PUT_ERR("error in check_overlap_cells(A,B) 5\n")
      if (!yac_check_overlap_cells(B,A)) PUT_ERR("error in check_overlap_cells(B,A) 5\n")
   }

   { // checks a simple problem
      //---------------
      // setup
      //---------------
      struct grid_cell A, B;
      double A_x [] = {0, 1, 1, 0};
      double A_y [] = {0, 0, 1, 1};
      double B_x [] = {0.5, 1.5, 1.5, 0.5};
      double B_y [] = {0.5, 0.5, 1.5, 1.5};

      for ( unsigned i = 0; i < 4; ++i ) {
         A_x [i] *= YAC_RAD;
         A_y [i] *= YAC_RAD;
         B_x [i] *= YAC_RAD;
         B_y [i] *= YAC_RAD;
      }

      enum yac_edge_type edge_type[4] = {LAT_CIRCLE, GREAT_CIRCLE,
                                         LAT_CIRCLE, GREAT_CIRCLE};

      A.coordinates_x = A_x;
      A.coordinates_y = A_y;
      A.coordinates_xyz = (double[4][3]){{-1}};
      A.num_corners = 4;
      A.edge_type = edge_type;
      B.coordinates_x = B_x;
      B.coordinates_y = B_y;
      B.coordinates_xyz = (double[4][3]){{-1}};
      B.num_corners = 4;
      B.edge_type = edge_type;

      generate_XYZ(A);
      generate_XYZ(B);

      //---------------
      // testing
      //---------------
      if (!yac_check_overlap_cells(A,A)) PUT_ERR("error in check_overlap_cells(A,A) 6\n")
      if (!yac_check_overlap_cells(B,B)) PUT_ERR("error in check_overlap_cells(B,B) 6\n")
      if (!yac_check_overlap_cells(A,B)) PUT_ERR("error in check_overlap_cells(A,B) 6\n")
      if (!yac_check_overlap_cells(B,A)) PUT_ERR("error in check_overlap_cells(B,A) 6\n")
   }

   return TEST_EXIT_CODE;
}

