/**
 * @file test_communicator_local_fail.c
 *
 * @copyright Copyright  (C)  2015 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 *
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <unistd.h>

#include "tests.h"

#include "communicator.h"
#include "communicator_local.h"

int test_index = -1;

static void
parse_options(int *argc, char ***argv);

int main (int argc, char **argv) {

   parse_options(&argc, &argv);

   { // tests with a single local rank
      struct communicator * comm;
      struct communicator * child_comm;

      comm = yac_local_comm_new();
      child_comm = yac_named_local_comm_new(comm, "child");

      { // tests the size of the communicator
         if (yac_get_comm_size(comm) != 1)
            return 0;
      }

      switch(test_index) {

        case 0: { // tests sending of data (first send then recv)
                  // send more than to receive -> should fail

           unsigned send_data[4] = {0,1,2,3};

           yac_comm_bsend(send_data, 4, COMM_UINT, 0, 0, comm);

           yac_comm_recv_callback(1, COMM_UINT, 0, 0, comm, send_data,
                                  NULL, NULL, MSG_PRIO_HIGH);

           yac_comm_wait(comm, 0);

           break;
        }

        case 1: { // tests sending of data (first recv then send)
                  // send more than to receive -> should fail

           unsigned send_data[4] = {0,1,2,3};

           yac_comm_recv_callback(1, COMM_UINT, 0, 0, comm, send_data,
                                  NULL, NULL, MSG_PRIO_HIGH);

           yac_comm_bsend(send_data, 4, COMM_UINT, 0, 0, comm);

           yac_comm_wait(comm, 0);

           break;
        }
      }

      yac_free_communicator(child_comm);
      yac_free_communicator(comm);
   }

   yac_comm_finalize();

   return TEST_EXIT_CODE;
}

static void
parse_options(int *argc, char ***argv)
{
  int opt;
  while ((opt = getopt(*argc, *argv, "t:")) != -1) {
    switch (opt) {
    case 't':
      test_index = atoi(optarg);
    }
  }
}

