!#define VERBOSE

!!
!! @file test_def_elements.F90
!!
!! @copyright Copyright  (C)  2013 DKRZ, MPI-M
!!
!! @version 1.0
!!
!! @author Moritz Hanke <hanke@dkrz.de>
!!         Rene Redler  <rene.redler@mpimet.mpg.de>
!!
!
! Keywords:
! Maintainer: Moritz Hanke <hanke@dkrz.de>
!             Rene Redler <rene.redler@mpimet.mpg.de>
! URL: https://doc.redmine.dkrz.de/YAC/html/index.html
!
! This file is part of YAC.
!
! Redistribution and use in source and binary forms, with or without
! modification, are  permitted provided that the following conditions are
! met:
!
! Redistributions of source code must retain the above copyright notice,
! this list of conditions and the following disclaimer.
!
! Redistributions in binary form must reproduce the above copyright
! notice, this list of conditions and the following disclaimer in the
! documentation and/or other materials provided with the distribution.
!
! Neither the name of the DKRZ GmbH nor the names of its contributors
! may be used to endorse or promote products derived from this software
! without specific prior written permission.
!
! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
! IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
! TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
! PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
! OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
! EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
! PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
! PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
! LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
! NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
! SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
!


#include "test_macros.inc"

program test_def_elements

  use utest
  use mo_yac_finterface, only : yac_fdef_comp, yac_fdef_subdomain, &
                                yac_fdef_elements, YAC_MAX_CHARLEN

  implicit none

  integer, parameter :: nbr_subdomains = 2
  integer, parameter :: nbr_vertices   = 6
  integer, parameter :: nbr_cells      = 2
  integer, parameter :: nbr_vertices_per_cell = 4
  integer, parameter :: nbr_connections = 7

  integer            :: nbr_vertices_per_cell_nu(nbr_cells)
  integer            :: cell_to_vertex_nu(nbr_connections)

  integer :: domain_id
  integer :: subdomain_id(nbr_subdomains)
  integer :: comp_id

  character(len=YAC_MAX_CHARLEN)  :: comp_name
  character(len=YAC_MAX_CHARLEN)  :: grid_name
  character(len=YAC_MAX_CHARLEN) :: xml_name
  character(len=YAC_MAX_CHARLEN) :: schema_name

#ifdef __use_REAL
  real :: x_vertices(nbr_vertices)
  real :: y_vertices(nbr_vertices)
#else
  double precision :: x_vertices(nbr_vertices)
  double precision :: y_vertices(nbr_vertices)
#endif

  integer  :: cell_to_vertex(nbr_vertices_per_cell,nbr_cells)

  integer  :: i
  logical  :: result

  xml_name    = 'coupling.xml'
  schema_name = 'coupling.xsd'
  call yac_finit ( xml_name , schema_name )

  call start_test("yac_fdef_elements")

  comp_id      = -99
  subdomain_id = -99

  comp_name = 'ICON-ocean'

  !
  ! Get a comp_id
  !
  call yac_fdef_comp ( comp_name, comp_id )

  !
  ! Get the subdomain_ids
  !

  do i = 1, nbr_subdomains

     grid_name = 'name1'

     call yac_fdef_subdomain ( comp_id,    &
                               grid_name,  &
                               subdomain_id(i) )

     result = subdomain_id(1) /= -99

     call test ( result )

#ifdef VERBOSE
     print *, ' yac_fdef_elements works with subdomain_id ', subdomain_id(i)
#endif

  enddo

  ! first subdomain

  i = 1

  !  1-------2
  !  |       |
  !  |       | cell 1
  !  |       |
  !  3-------4   Eq.
  !  |       |
  !  |       | cell 2
  !  |       |
  !  5-------6

  x_vertices(1) = -0.5; y_vertices(1) =  1.0
  x_vertices(2) =  0.5; y_vertices(2) =  1.0
  x_vertices(3) = -0.5; y_vertices(3) =  0.0
  x_vertices(4) =  0.5; y_vertices(4) =  0.0
  x_vertices(5) = -0.5; y_vertices(5) = -1.0
  x_vertices(6) =  0.5; y_vertices(6) = -1.0

  cell_to_vertex(1,1) = 1
  cell_to_vertex(2,1) = 3
  cell_to_vertex(3,1) = 4
  cell_to_vertex(4,1) = 2
  cell_to_vertex(1,2) = 3
  cell_to_vertex(2,2) = 5
  cell_to_vertex(3,2) = 6
  cell_to_vertex(4,2) = 4

  call yac_fdef_elements ( subdomain_id(i),       &
                           nbr_vertices,          &
                           nbr_cells,             &
                           nbr_vertices_per_cell, &
                           x_vertices,            &
                           y_vertices,            &
                           cell_to_vertex )

  ! second subdomain

  i = 2

  !      1
  !     / \
  !    /   \   cell 1
  !   /     \
  !  2-------3   Eq.
  !  |       |
  !  |       |  cell 2
  !  |       |
  !  4-------5

  x_vertices(1) =  0.0; y_vertices(1) =  1.0
  x_vertices(2) = -0.5; y_vertices(2) =  0.0
  x_vertices(3) =  0.5; y_vertices(3) =  0.0
  x_vertices(4) = -0.5; y_vertices(4) = -1.0
  x_vertices(5) =  0.5; y_vertices(5) = -1.0

  cell_to_vertex_nu(1) = 1
  cell_to_vertex_nu(2) = 2
  cell_to_vertex_nu(3) = 3
  cell_to_vertex_nu(4) = 2
  cell_to_vertex_nu(5) = 4
  cell_to_vertex_nu(6) = 5
  cell_to_vertex_nu(7) = 3

  nbr_vertices_per_cell_nu(1) = 3
  nbr_vertices_per_cell_nu(2) = 4

  call yac_fdef_elements ( subdomain_id(i),          &
                           nbr_vertices,             &
                           nbr_cells,                &
                           nbr_connections,          &
                           nbr_vertices_per_cell_nu, &
                           x_vertices,               &
                           y_vertices,               &
                           cell_to_vertex_nu )

  ! Finally, connect the subdomains

  domain_id = 0

  call yac_fconnect_subdomains ( comp_id, nbr_subdomains, subdomain_id, domain_id )  
 
#ifdef VERBOSE
  print *, " yac_fconnect_subdomains worked with domain_id ", domain_id
#endif

  result = ( domain_id /= -99 )

  call test ( result )

  call stop_test

  call exit_tests

  call yac_ffinalize ( )

end program test_def_elements
