!!
!! @file test_def_grid.F90
!!
!! @copyright Copyright  (C)  2013 DKRZ, MPI-M
!!
!! @version 1.0
!!
!! @author Moritz Hanke <hanke@dkrz.de>
!!         Rene Redler  <rene.redler@mpimet.mpg.de>
!!
!
! Keywords:
! Maintainer: Moritz Hanke <hanke@dkrz.de>
!             Rene Redler <rene.redler@mpimet.mpg.de>
! URL: https://doc.redmine.dkrz.de/YAC/html/index.html
!
! This file is part of YAC.
!
! Redistribution and use in source and binary forms, with or without
! modification, are  permitted provided that the following conditions are
! met:
!
! Redistributions of source code must retain the above copyright notice,
! this list of conditions and the following disclaimer.
!
! Redistributions in binary form must reproduce the above copyright
! notice, this list of conditions and the following disclaimer in the
! documentation and/or other materials provided with the distribution.
!
! Neither the name of the DKRZ GmbH nor the names of its contributors
! may be used to endorse or promote products derived from this software
! without specific prior written permission.
!
! THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
! IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
! TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
! PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
! OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
! EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
! PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
! PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
! LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
! NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
! SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
!


#include "test_macros.inc"

program test_def_subdomain

  use utest
  use mo_yac_finterface, only : yac_fdef_comp, &
                                yac_fdef_subdomain, YAC_MAX_CHARLEN
  implicit none

  integer :: comp_id
  integer :: subdomain_id(2)

  integer, parameter :: ndim = 3

  character(len=YAC_MAX_CHARLEN) :: comp_name
  character(len=YAC_MAX_CHARLEN) :: grid_name
  character(len=YAC_MAX_CHARLEN) :: xml_name
  character(len=YAC_MAX_CHARLEN) :: schema_name

  logical  :: result

  call start_test("def_subdomain")

  subdomain_id = -99

  xml_name    = 'coupling.xml'
  schema_name = 'coupling.xsd'
  call yac_finit ( xml_name , schema_name )

  comp_name   = 'ICON-ocean'
  call yac_fdef_comp ( comp_name, comp_id )

#ifdef VERBOSE
  print *, ' def_comp returned comp_id ', comp_id
#endif

  grid_name = 'name1'

  call yac_fdef_subdomain ( comp_id,    &
                            grid_name,  &
                            subdomain_id(1) )

  result = ( subdomain_id(1) /= -99 )

  call test ( result )

#ifdef VERBOSE
  print *, ' def_subdomain returned subdomain_id ', subdomain_id(1)
#endif

  call yac_fdef_subdomain ( comp_id,    &
                            grid_name,  &
                            subdomain_id(2) )

  result = ( subdomain_id(2) /= -99 )

  call test ( result )

#ifdef VERBOSE
  print *, ' def_subdomain returned subdomain_id ', subdomain_id(2)
#endif

  call yac_ffinalize ( );

  call stop_test

  call exit_tests

end program test_def_subdomain
