/**
 * @file test_find_overlap.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "tests.h"
#include "search.h"
#include "grid.h"
#include "grid_reg2d.h"
#include "test_common.h"

#include <string.h>

int main (void) {

   //---------------
   // setup
   //---------------

   struct grid * reg_tgt_grid;
   struct grid * reg_src_grid;

   double coordinates_x[2][3] = {{0,1,2}, {0.5,1.5,2.5}};
   double coordinates_y[2][3] = {{0,1,2}, {0.5,1.5,2.5}};
   unsigned num_cells[2] = {2,2};
   unsigned cyclic[2] = {0,0};

   size_t initial_num_deps_per_element[4] = {1,1,1,1};
   size_t * p_initial_num_deps_per_element;
   unsigned initial_dependencies[4] = {0,0,0,0};
   unsigned * p_initial_dependencies;

   size_t ref_num_overlap_per_cell[4] = {1,2,2,4};
   unsigned ref_overlaps[4][4] = {{0,-1,-1,-1},
                                  {0,1,-1,-1},
                                  {0,2,-1,-1},
                                  {0,1,2,3}};

   // set up grid data
   reg_src_grid = yac_reg2d_grid_new(coordinates_x[0], coordinates_y[0], num_cells, cyclic);
   reg_tgt_grid = yac_reg2d_grid_new(coordinates_x[1], coordinates_y[1], num_cells, cyclic);

   // set up initial_src_to_tgt
   p_initial_num_deps_per_element = TO_POINTER(initial_num_deps_per_element);
   p_initial_dependencies = TO_POINTER(initial_dependencies);
   struct dep_list * initial_src_to_tgt =
      yac_dep_list_new(
         4, p_initial_num_deps_per_element, p_initial_dependencies);

   //---------------
   // testing
   //---------------

   struct dep_list * src_to_tgt =
      yac_find_overlapping_cells(
         reg_src_grid, reg_tgt_grid, initial_src_to_tgt);

   if (yac_get_num_elements(src_to_tgt) != 4)
      PUT_ERR("wrong number of elements in src_to_tgt\n")

   for (size_t i = 0; i < 4; ++i) {

      size_t num_matching_overlaps = 0;
      unsigned const * curr_overlaps =
         yac_get_dependencies_of_element(src_to_tgt, i);
      size_t curr_num_overlaps =
         yac_get_num_dependencies_of_element(src_to_tgt, i);

      if (curr_num_overlaps != ref_num_overlap_per_cell[i])
         PUT_ERR("wrong number of overlaps per cell in src_to_tgt\n")

      for (size_t j = 0; j < ref_num_overlap_per_cell[i]; ++j)
         for (size_t k = 0; k < ref_num_overlap_per_cell[i]; ++k)
            if (curr_overlaps[j] == ref_overlaps[i][k])
               ++num_matching_overlaps;

      if (num_matching_overlaps != ref_num_overlap_per_cell[i])
         PUT_ERR("wrong overlaps in src_to_tgt\n")
   }

   yac_dep_list_delete(src_to_tgt);
   yac_dep_list_delete(initial_src_to_tgt);
   yac_delete_grid(reg_src_grid);
   yac_delete_grid(reg_tgt_grid);

   return TEST_EXIT_CODE;
}

