/**
 * @file test_interpolation_method_common.c
 *
 * @copyright Copyright  (C)  2018 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *         Rene Redler  <rene.redler@mpimet.mpg.de>
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "tests.h"
#include "test_interpolation_method_common.h"

void check_data_request(
  struct interpolation_method_data_request * request,
  struct interpolation_method_data_request * ref_request,
  unsigned num_points_per_subdomain) {

  for (unsigned i = 0; i < num_points_per_subdomain; ++i) {

    if (request[i].num_values != ref_request[i].num_values)  {
      PUT_ERR("wrong number of requested points\n");
      printf("check_data_request subdomain %u : num_values %u != %u\n",
             i, request[i].num_values, ref_request[i].num_values);
    }

    for (unsigned j = 0; j < ref_request[i].num_values; ++j) {

       if (request[i].global_ids[j] != ref_request[i].global_ids[j]) {
          PUT_ERR("wrong global_id requested\n");
          printf("check_data_request subdomain %u global_id %u : %d %d\n",
                 i, j, request[i].global_ids[j], ref_request[i].global_ids[j]);
       }

       if (request[i].ranks[j] != ref_request[i].ranks[j]) {
          PUT_ERR("point from wrong rank requested\n");
          printf("check_data_request subdomain %u rank %u : %d %d\n",
                 i, j, request[i].ranks[j], ref_request[i].ranks[j]);
       }
    }
  }
}

void delete_interpolation_method_data_request_stack(
  struct interpolation_method_data_request * request,
  unsigned num_points_per_subdomain) {

  if (request != NULL) {

    for (unsigned i = 0; i < num_points_per_subdomain; ++i) {
      free(request[i].global_ids);
      free(request[i].ranks);
    }
  }
}

