#if 1
#define TEST1
#define TEST2
#define TEST3
#define TEST4
#define TEST5
#define TEST6
#define TEST7
#define TEST8
#endif
/**
 * @file test_interpolation_method_spmap.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "tests.h"

#include "grid.h"
#include "grid_reg2d.h"
#include "communicator.h"
#include "communicator_local.h"
#include "domain.h"
#include "global_search.h"
#include "interpolation_method_spmap.h"
#include "utils.h"
#include "test_interpolation_method_common.h"

int main (void) {

   static double tol = 1.0e-14;

#ifdef TEST1
   {  // 1st spmap recovery with one source and one target process

      // the global source domain is a 2x2 grid:
      //       06--10--07--11--08
      //       |       |       |
      //       06  02  08  03  09
      //       |       |       |
      //       03--05--04--07--05
      //       |       |       |
      //       01  00  03  01  04
      //       |       |       |
      //       00--00--01--02--02
      //
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes
      struct communicator * comm_src, * comm_tgt;

      comm_src = yac_local_comm_new();
      comm_tgt = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src, * comm_named_tgt;

      comm_named_src = yac_named_local_comm_new(comm_src, "src");
      comm_named_tgt = yac_named_local_comm_new(comm_tgt, "tgt");

      // define domains on source

      struct grid * grid_src;
      unsigned num_cells_src[2] = {2,2};
      unsigned cyclic_src[2] = {0,0};
      double coordinates_x_src[3] = {-1,0,1};
      double coordinates_y_src[3] = {-1,0,1};

      for ( unsigned i = 0; i < 3; ++i ) {
         coordinates_x_src[i] *= YAC_RAD;
         coordinates_y_src[i] *= YAC_RAD;
      }

      double coordinates_x_src_points[2] = {-0.5*YAC_RAD, 0.5*YAC_RAD};
      double coordinates_y_src_points[2] = {-0.5*YAC_RAD, 0.5*YAC_RAD};

      grid_src = yac_reg2d_grid_new(coordinates_x_src, coordinates_y_src, num_cells_src, cyclic_src);

      struct subdomain subdomain_src;

      int global_corner_ids_src[9] = {0,1,2, 3,4,5, 6,7,8};
      unsigned corner_is_on_rank_src[9] = {-1,-1,-1, -1,-1,-1, -1,-1,-1};
      int global_cell_ids_src[4] = {0,1,2,3};
      unsigned cell_is_on_rank_src[4] = {-1,-1,-1,-1};

      yac_init_subdomain_unstruct ( &subdomain_src, grid_src, comm_named_src,
                                    global_corner_ids_src, corner_is_on_rank_src,
                                    global_cell_ids_src, cell_is_on_rank_src,
                                    NULL, NULL );

      struct domain l_domain_src;

      struct subdomain * blocks_src = &subdomain_src;

      yac_init_domain(&l_domain_src, comm_named_src, &blocks_src, 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt;

      unsigned num_cells_tgt[2] = {1,1};
      unsigned cyclic_tgt[2] = {0,0};

      double coordinates_x_tgt[2] = {-0.5*YAC_RAD,0.5*YAC_RAD};
      double coordinates_y_tgt[2] = {-0.5*YAC_RAD,0.5*YAC_RAD};

      double coordinates_xyz_tgt_point[1][3];
      LLtoXYZ_deg(0.0, 0.0, coordinates_xyz_tgt_point[0]);

      grid_tgt = yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt, num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt;

      int global_corner_ids_tgt[4] = {0,1,2,3};
      unsigned corner_is_on_rank_tgt[4] = {-1,-1,-1,-1};
      int global_cell_ids_tgt[1] = {0};
      unsigned cell_is_on_rank_tgt[1] = {-1};

      yac_init_subdomain_unstruct( &subdomain_tgt, grid_tgt, comm_named_tgt,
                                   global_corner_ids_tgt, corner_is_on_rank_tgt,
                                   global_cell_ids_tgt, cell_is_on_rank_tgt,
                                   NULL, NULL );

      struct domain l_domain_tgt;

      struct subdomain * blocks_tgt = {&subdomain_tgt};

      yac_init_domain(&l_domain_tgt, comm_named_tgt, &blocks_tgt, 1, "domain_tgt");

      // finish domain definition
      yac_enddef_domain(&l_domain_src);
      yac_enddef_domain(&l_domain_tgt);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[2];

      search[0] = yac_global_search_new(&l_domain_src, "domain_tgt", comm_src,
                                        1,0, "component_p1", "component_p2");
      search[1] = yac_global_search_new(&l_domain_tgt, "domain_src", comm_tgt,
                                        0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      yac_enddef_global_search(search[0]);
      yac_enddef_global_search(search[1]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src;
      yac_init_points(&cell_points_src, grid_src, CELL,
                      coordinates_x_src_points, coordinates_y_src_points);

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain = &cell_points_src;
      struct points ** point_sets = &points_per_subdomain;

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap;

      method_spmap =
          yac_new_interpolation_method_spmap (search[0], &l_domain_src,
                                             "spmap stack", &point_sets, 1, 0);

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[2] = {method_spmap, NULL};
      struct interpolation_plan_element * interpolation_plan_element_stack[2] = {NULL, NULL};
      int global_ids[1] = {0};
      unsigned num_points = 1;
      unsigned num_interpolated_points[1];

      yac_do_search_interpolation_method ( method_stack,
                                           interpolation_plan_element_stack, global_ids,
                                           coordinates_xyz_tgt_point, CELL, num_points,
                                           num_interpolated_points, 1 );

      yac_enddef_interpolation_method(method_stack);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[4] = {1,2,3,4};
      double * subdomain_in_data[1] = {in_data_raw};
      double ** in_data[2] = {NULL, subdomain_in_data};
      double send_buffer[1];

      yac_execute_interpolation_plan_element ( interpolation_plan_element_stack,
                                               in_data, send_buffer);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_;
      struct interpolation_method_data_request * data_request = &data_request_;

      yac_get_source_exchange_data_interpolation_method ( method_stack, &data_request, 1 );

      struct interpolation_method_data_request ref_data_request = {0, NULL, NULL};

      check_data_request(data_request, &ref_data_request, 1);

      unsigned ref_num_interpolated_points = 1;
      int ref_global_ids[1] = {0};
      double ref_out_data[1] = {10};

      if (ref_num_interpolated_points != num_interpolated_points[0]) {
          printf ( "1st) wrong number of interpolated points %u %u \n",
                    ref_num_interpolated_points, num_interpolated_points[0] );
          PUT_ERR( "1st) wrong number of interpolated points\n");
      }
      if (ref_global_ids[0] != global_ids[0]) {
          printf ( "1st) wrong global id order %d %d \n", ref_global_ids[0], global_ids[0] );
          PUT_ERR( "1st) wrong global id order\n");
      }
      if ( fabs(send_buffer[0]-ref_out_data[0]) > tol ) {
          printf ( "1st) send_data %f; reference %f \n", send_buffer[0], ref_out_data[0] );
          PUT_ERR( "1st) wrong result buffer\n");
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned i = 0; i < sizeof(method_stack) /
      sizeof(method_stack[0]); ++i)
        if (method_stack[i] != NULL)
          yac_delete_interpolation_method(method_stack[i]);
      for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack) /
      sizeof(interpolation_plan_element_stack[0]); ++i) {
          yac_free_interpolation_plan_element(interpolation_plan_element_stack[i]);
          free(interpolation_plan_element_stack[i]);
      }

      delete_interpolation_method_data_request_stack(data_request, 1);

      yac_free_points(&cell_points_src);

      yac_global_search_delete(search[0]);
      yac_global_search_delete(search[1]);

      yac_free_domain(&l_domain_src);
      yac_free_domain(&l_domain_tgt);

      yac_delete_grid(grid_src);
      yac_delete_grid(grid_tgt);

      yac_free_communicator(comm_src);
      yac_free_communicator(comm_tgt);

      yac_free_communicator(comm_named_src);
      yac_free_communicator(comm_named_tgt);

      yac_reset_interpolation_method();
   }
#endif
#ifdef TEST2
   {  // 2nd spmap recovery with two source and one target processes

      // the global source domain is a 2x2 grid:
      //
      //       06--10--07--11--08
      //       |       |       |
      //       06  02  08  03  09
      //       |       |       |
      //       03--05--04--07--05
      //       |       |       |
      //       01  00  03  01  04
      //       |       |       |
      //       00--00--01--02--02
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes
      struct communicator * comm_src[2], * comm_tgt;

      comm_src[0] = yac_local_comm_new();
      comm_src[1] = yac_local_comm_new();
      comm_tgt = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src[2], * comm_named_tgt;

      comm_named_src[0] = yac_named_local_comm_new(comm_src[0], "src");
      comm_named_src[1] = yac_named_local_comm_new(comm_src[1], "src");
      comm_named_tgt = yac_named_local_comm_new(comm_tgt, "tgt");

      // define domains on source

      struct grid * grid_src[2];
      unsigned num_cells_src[2] = {2,2};
      unsigned cyclic_src[2] = {0,0};
      double coordinates_x_src[3] = {-1,0,1};
      double coordinates_y_src[3] = {-1,0,1};

      for ( unsigned i = 0; i < 3; ++i ) {
         coordinates_x_src[i] *= YAC_RAD;
         coordinates_y_src[i] *= YAC_RAD;
      }

      double coordinates_x_src_points[2] = {-0.5*YAC_RAD, 0.5*YAC_RAD};
      double coordinates_y_src_points[2] = {-0.5*YAC_RAD, 0.5*YAC_RAD};

      grid_src[0] = yac_reg2d_grid_new(coordinates_x_src, coordinates_y_src,
                                       num_cells_src, cyclic_src);

      grid_src[1] = yac_reg2d_grid_new(coordinates_x_src, coordinates_y_src,
                                       num_cells_src, cyclic_src);

      struct subdomain subdomain_src[2];

      int global_corner_ids_src[9] = {0,1,2, 3,4,5, 6,7,8};
      unsigned corner_is_on_rank_src[2][9] = {{-1,-1, 1, -1,-1, 1, -1,-1, 1},
                     { 0,-1,-1,  0,-1,-1,  0,-1,-1}};
      int global_cell_ids_src[4] = {0,1,2,3};
      unsigned cell_is_on_rank_src[2][4] = {{-1, 1,-1, 1},
                   { 0,-1, 0,-1}};

      yac_init_subdomain_unstruct ( &subdomain_src[0], grid_src[0], comm_named_src[0],
                                    global_corner_ids_src, corner_is_on_rank_src[0],
                                    global_cell_ids_src, cell_is_on_rank_src[0],
                                    NULL, NULL );

      yac_init_subdomain_unstruct ( &subdomain_src[1], grid_src[1], comm_named_src[1],
                                    global_corner_ids_src, corner_is_on_rank_src[1],
                                    global_cell_ids_src, cell_is_on_rank_src[1],
                                    NULL, NULL );

      struct domain l_domain_src[2];

      struct subdomain * blocks_src[2] = {&subdomain_src[0],&subdomain_src[1]};

      yac_init_domain(&l_domain_src[0], comm_named_src[0], &blocks_src[0], 1, "domain_src");
      yac_init_domain(&l_domain_src[1], comm_named_src[1], &blocks_src[1], 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt;
      unsigned num_cells_tgt[2] = {1,1};
      unsigned cyclic_tgt[2] = {0,0};
      double coordinates_x_tgt[5] = {-0.5,0.5};
      double coordinates_y_tgt[3] = {-0.5,0.5};

      double coordinates_xyz_tgt_point[1][3];
      LLtoXYZ_deg(0.0, 0.0, coordinates_xyz_tgt_point[0]);

      for ( unsigned i = 0; i < 2; ++i ) {
         coordinates_x_tgt[i] *= YAC_RAD;
         coordinates_y_tgt[i] *= YAC_RAD;
      }

      grid_tgt = yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt, num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt;

      int global_corner_ids_tgt[4] = {0,1,2,3};
      unsigned corner_is_on_rank_tgt[4] = {-1,-1,-1,-1};
      int global_cell_ids_tgt[1] = {0};
      unsigned cell_is_on_rank_tgt[1] = {-1};

      yac_init_subdomain_unstruct ( &subdomain_tgt, grid_tgt, comm_named_tgt,
                                    global_corner_ids_tgt, corner_is_on_rank_tgt,
                                    global_cell_ids_tgt, cell_is_on_rank_tgt,
                                    NULL, NULL );

      struct domain l_domain_tgt;

      struct subdomain * blocks_tgt = {&subdomain_tgt};

      yac_init_domain(&l_domain_tgt, comm_named_tgt, &blocks_tgt, 1, "domain_tgt");

      // finish domain definition
      yac_enddef_domain(&l_domain_src[0]);
      yac_enddef_domain(&l_domain_src[1]);
      yac_enddef_domain(&l_domain_tgt);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[3];

      search[0] = yac_global_search_new(&l_domain_src[0], "domain_tgt", comm_src[0],
                                        1,0, "component_p1", "component_p2");
      search[1] = yac_global_search_new(&l_domain_src[1], "domain_tgt", comm_src[1],
                                        1,0, "component_p1", "component_p2");
      search[2] = yac_global_search_new(&l_domain_tgt, "domain_src", comm_tgt,
                                        0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      yac_enddef_global_search(search[0]);
      yac_enddef_global_search(search[1]);
      yac_enddef_global_search(search[2]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src[2];
      yac_init_points(&cell_points_src[0], grid_src[0], CELL,
                      coordinates_x_src_points, coordinates_y_src_points);
      yac_init_points(&cell_points_src[1], grid_src[1], CELL,
                      coordinates_x_src_points, coordinates_y_src_points);

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain[2] = {&cell_points_src[0],
                                                 &cell_points_src[1]};
      struct points ** point_sets[2] = {&points_per_subdomain[0],
                                        &points_per_subdomain[1]};

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap[2];

      for (unsigned i = 0; i < 2; ++i)
        method_spmap[i] =
           yac_new_interpolation_method_spmap (search[i], &l_domain_src[i],
                                              "spmap stack", &point_sets[i], 1, 0);

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[2][2] = {{method_spmap[0], NULL},{method_spmap[1], NULL}};
      struct interpolation_plan_element * interpolation_plan_element_stack[2][2] =
                                                         {{NULL, NULL},{NULL, NULL}};
      int global_ids[2][1] = {{0},{0}};
      unsigned num_points = 1;
      unsigned num_interpolated_points[2][1];

      yac_do_search_interpolation_method(method_stack[0], interpolation_plan_element_stack[0],
                                         global_ids[0], coordinates_xyz_tgt_point, CELL,
                                         num_points, num_interpolated_points[0], 1);

      yac_do_search_interpolation_method(method_stack[1], interpolation_plan_element_stack[1],
                                         global_ids[1], coordinates_xyz_tgt_point, CELL,
                                         num_points, num_interpolated_points[1], 1);

      yac_enddef_interpolation_method(method_stack[0]);
      yac_enddef_interpolation_method(method_stack[1]);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[2][4] = {{ 1,-1, 3,-1},
                                  { 2, 4}};
      double * remote_data[1] = {in_data_raw[1]};
      double * subdomain_in_data[1] = {in_data_raw[0]};
      double ** in_data[2][2] = {{remote_data, subdomain_in_data},{NULL, NULL}};
      double send_buffer[2][1] = {{-1},{-1}};

      yac_execute_interpolation_plan_element(interpolation_plan_element_stack[0],
                                             in_data[0], send_buffer[0]);
      yac_execute_interpolation_plan_element(interpolation_plan_element_stack[1],
                                             in_data[1], send_buffer[1]);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_[2];
      struct interpolation_method_data_request * data_request[2] =
         {&data_request_[0],&data_request_[1]};

      yac_get_source_exchange_data_interpolation_method(method_stack[0], &data_request[0], 1);
      yac_get_source_exchange_data_interpolation_method(method_stack[1], &data_request[1], 1);

      int ref_requested_global_ids[2] = {1,3};
      int ref_requested_ranks[2] = {1,1};

      struct interpolation_method_data_request ref_data_request[2] =
         {{2, ref_requested_global_ids,  ref_requested_ranks}, {0,NULL,NULL}};

      unsigned ref_num_interpolated_points[2] = {1,0};
      int ref_global_ids[1] = {0};
      double ref_out_data[2] = {10,-1};

      for (unsigned j = 0; j < 2; ++j ) {

         check_data_request(data_request[j], &ref_data_request[j], 1);

         if (ref_num_interpolated_points[j] != num_interpolated_points[j][0]) {
            printf ( "2nd) wrong number of interpolated points %u: %u %u \n",
                           j, ref_num_interpolated_points[j], num_interpolated_points[j][0] );
            PUT_ERR( "2nd) wrong number of interpolated points\n");
         }
         if (ref_global_ids[0] != global_ids[j][0]) {
            printf ( "2nd) wrong global id order %u: %d %d \n",
                           j, ref_global_ids[0], global_ids[j][0] );
            PUT_ERR( "2nd) wrong global id order\n");
         }
         if ( fabs(send_buffer[j][0] - ref_out_data[j] ) > tol ) {
            printf ( "2nd) results at (%u,0) are %f %f \n",
                          j, send_buffer[j][0], ref_out_data[j] );
            PUT_ERR( "2nd) wrong result buffer\n");
         }
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned j = 0; j < 2; ++j ) {
         for (unsigned i = 0; i < sizeof(method_stack[j])/sizeof(method_stack[j][0]); ++i)
            if (method_stack[j][i] != NULL)
               yac_delete_interpolation_method(method_stack[j][i]);
         for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack[j]) /
              sizeof(interpolation_plan_element_stack[j][0]); ++i) {
            yac_free_interpolation_plan_element(interpolation_plan_element_stack[j][i]);
            free(interpolation_plan_element_stack[j][i]);
         }
         delete_interpolation_method_data_request_stack(data_request[j], 1);
      }

      yac_free_points(&cell_points_src[0]);
      yac_free_points(&cell_points_src[1]);

      yac_global_search_delete(search[0]);
      yac_global_search_delete(search[1]);
      yac_global_search_delete(search[2]);

      yac_free_domain(&l_domain_src[0]);
      yac_free_domain(&l_domain_src[1]);
      yac_free_domain(&l_domain_tgt);

      yac_delete_grid(grid_src[0]);
      yac_delete_grid(grid_src[1]);
      yac_delete_grid(grid_tgt);

      yac_free_communicator(comm_src[0]);
      yac_free_communicator(comm_src[1]);
      yac_free_communicator(comm_tgt);

      yac_free_communicator(comm_named_src[1]);
      yac_free_communicator(comm_named_src[0]);
      yac_free_communicator(comm_named_tgt);

      yac_reset_interpolation_method();
   }
#endif
#ifdef TEST3
   {  // 3rd spmap recovery with three source and two target processes

      // the global source domain is a 3x3 grid:
      //
      //       12--21--13--22--14--23--15
      //       |       |       |       |
      //       15  06  17  07  19  08  20
      //       |       |       |       |
      //       08--14--09--16--10--18--11
      //       |       |       |       |
      //       08  03  10  04  12  05  13
      //       |       |       |       |
      //       04--07--05--09--06--11--07
      //       |       |       |       |
      //       01  00  03  01  05  02  06
      //       |       |       |       |
      //       00--00--01--02--02--04--03
      //
      // the global target domain is a 2x2 grid:
      //
      //       06--10--07--11--08
      //       |       |       |
      //       06  02  08  03  09
      //       |       |       |
      //       03--05--04--07--05
      //       |       |       |
      //       01  00  03  01  04
      //       |       |       |
      //       00--00--01--02--02
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes

      #define NUM_SRC 3
      #define NUM_TGT 2

      struct communicator * comm_src[NUM_SRC], * comm_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_src[i] = yac_local_comm_new();

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_tgt[i] = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src[NUM_SRC], * comm_named_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_named_src[i] = yac_named_local_comm_new(comm_src[i], "src");

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_named_tgt[i] = yac_named_local_comm_new(comm_tgt[i], "tgt");

      // define domains on source

      struct grid * grid_src[NUM_SRC];

      unsigned num_cells_src[NUM_SRC][2] = {{2,3}, {3,3}, {2,3}};

      unsigned cyclic_src[2] = {0,0};

      double coordinates_x_src[NUM_SRC][4] = {{-1.5,-0.5,0.5},
                                              {-1.5,-0.5,0.5,1.5},
                                              {-0.5,0.5,1.5}};
      double coordinates_y_src[4] = {-1.5,-0.5,0.5,1.5};

      double coordinates_x_src_points[NUM_SRC][3] = {{-1.0*YAC_RAD, 0.0},
                                                     {-1.0*YAC_RAD, 0.0, 1.0*YAC_RAD},
                                                     { 0.0, 1.0*YAC_RAD}};

      double coordinates_y_src_points[3] = {-1.0*YAC_RAD, 0.0, 1.0*YAC_RAD};

      for ( unsigned j = 0; j < NUM_SRC; ++j )
         for ( unsigned i = 0; i < 4; ++i )
            coordinates_x_src[j][i] *= YAC_RAD;

      for ( unsigned i = 0; i < 4; ++i )
         coordinates_y_src[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_SRC; ++i)
         grid_src[i] = yac_reg2d_grid_new(coordinates_x_src[i], coordinates_y_src,
               num_cells_src[i], cyclic_src);

      struct subdomain subdomain_src[NUM_SRC];

      int global_corner_ids_src[NUM_SRC][16] = {{0,1,2, 4,5,6, 8,9,10, 12,13,14},
                                                {0,1,2,3, 4,5,6,7, 8,9,10,11, 12,13,14,15},
                                                {1,2,3, 5,6,7, 9,10,11, 13,14,15}};

      unsigned corner_is_on_rank_src[NUM_SRC][16] = {{-1,-1,1, -1,-1,1, -1,-1,1, -1,-1,1},
                                                     {0,-1,-1,2, 0,-1,-1,2, 0,-1,-1,2, 0,-1,-1,2},
                                                     {1,-1,-1, 1,-1,-1, 1,-1,-1, 1,-1,-1}};

      int global_cell_ids_src[NUM_SRC][9] = {{0,1,3,4,6,7},
                                             {0,1,2,3,4,5,6,7,8},
                                             {1,2,4,5,7,8}};

      unsigned cell_is_on_rank_src[NUM_SRC][9] = {{-1,1,-1,1,-1,1},
                                                  {0,-1,2,0,-1,2,0,-1,2},
                                                  {1,-1,1,-1,1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_subdomain_unstruct(&subdomain_src[i], grid_src[i], comm_named_src[i],
                                    global_corner_ids_src[i], corner_is_on_rank_src[i],
                                    global_cell_ids_src[i], cell_is_on_rank_src[i],
                                    NULL, NULL);

      struct domain l_domain_src[NUM_SRC];

      struct subdomain * blocks_src[NUM_SRC] = {&subdomain_src[0],
                                                &subdomain_src[1],
                                                &subdomain_src[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_domain(&l_domain_src[i], comm_named_src[i], &blocks_src[i], 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt[NUM_TGT];
      unsigned num_cells_tgt[2] = {2,2};
      unsigned cyclic_tgt[2] = {0,0};
      double coordinates_x_tgt[3] = {-1,0,1};
      double coordinates_y_tgt[3] = {-1,0,1};

      double coordinates_x_tgt_point[2] = {-0.5,0.5};
      double coordinates_y_tgt_point[2] = {-0.5,0.5};

      for ( unsigned i = 0; i < 3; ++i ) {
        coordinates_x_tgt[i] *= YAC_RAD;
        coordinates_y_tgt[i] *= YAC_RAD;
      }

      for ( unsigned i = 0; i < 2; ++i ) {
        coordinates_x_tgt_point[i] *= YAC_RAD;
        coordinates_y_tgt_point[i] *= YAC_RAD;
      }

      for (unsigned i = 0; i < NUM_TGT; ++i)
        grid_tgt[i] = yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt,
                                     num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt[NUM_TGT];

      int global_corner_ids_tgt[9] = {0,1,2,3,4,5,6,7,8};
      unsigned corner_is_on_rank_tgt[NUM_TGT][9] = {{-1,-1,-1, -1,-1,-1, 1,1,1},
                                                    {0,0,0, -1,-1,-1, -1,-1,-1}};
      int global_cell_ids_tgt[4] = {0,1,2,3};
      unsigned cell_is_on_rank_tgt[NUM_TGT][4] = {{-1,-1,1,1},{0,0,-1,-1}};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_subdomain_unstruct(&subdomain_tgt[i], grid_tgt[i], comm_named_tgt[i],
                                    global_corner_ids_tgt, corner_is_on_rank_tgt[i],
                                    global_cell_ids_tgt, cell_is_on_rank_tgt[i],
                                    NULL, NULL);

      struct domain l_domain_tgt[NUM_TGT];

      struct subdomain * blocks_tgt[NUM_TGT] = {&subdomain_tgt[0],
                                                &subdomain_tgt[1]};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_domain(&l_domain_tgt[i], comm_named_tgt[i], &blocks_tgt[i], 1, "domain_tgt");

      // finish domain definition
      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_enddef_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_enddef_domain(&l_domain_tgt[i]);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[NUM_SRC + NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        search[i] = yac_global_search_new(&l_domain_src[i], "domain_tgt", comm_src[i],
                                          1,0, "component_p1", "component_p2");
      for (unsigned i = 0; i < NUM_TGT; ++i)
        search[NUM_SRC + i] =
          yac_global_search_new(&l_domain_tgt[i], "domain_src", comm_tgt[i],
                                0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
        yac_enddef_global_search(search[i]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_init_points(&cell_points_src[i], grid_src[i], CELL,
                         coordinates_x_src_points[i], coordinates_y_src_points );

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain[NUM_SRC] = {&cell_points_src[0],
                                                       &cell_points_src[1],
                                                       &cell_points_src[2]};
      struct points ** point_sets[NUM_SRC] = {&points_per_subdomain[0],
                                              &points_per_subdomain[1],
                                              &points_per_subdomain[2]};

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        method_spmap[i] =
                yac_new_interpolation_method_spmap (search[i], &l_domain_src[i],
                                                   "spmap stack", &point_sets[i], 1, 0);

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[NUM_SRC][2] = {{method_spmap[0], NULL},
                                                                {method_spmap[1], NULL},
                                                                {method_spmap[2], NULL}};

      struct interpolation_plan_element * interpolation_plan_element_stack[NUM_SRC][2] =
         {{NULL, NULL},{NULL, NULL},{NULL, NULL}};
      int global_ids[NUM_SRC][4] = {{0,1,2,3},{0,1,2,3},{0,1,2,3}};
      double coordinates_xyz[NUM_SRC][4][3];
      unsigned num_points = 4;
      unsigned num_interpolated_points[NUM_SRC][1];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        for (unsigned j = 0; j < num_points; ++j)
          LLtoXYZ(coordinates_x_tgt_point[j&1],
                  coordinates_y_tgt_point[j>>1],
                  coordinates_xyz[i][j]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_do_search_interpolation_method(
            method_stack[i], interpolation_plan_element_stack[i], global_ids[i],
            coordinates_xyz[i], CELL, num_points,
            num_interpolated_points[i], 1);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_enddef_interpolation_method(method_stack[i]);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[NUM_SRC][2][9] = {{{-1},{1,-1,4,-1,7,-1}},
                                           {{3,6,9},{-1,2,-1,-1,5,-1,-1,8,-1}},
                                           {{-1},{-1,3,-1,6,-1,9}}};
      double * remote_data[NUM_SRC][1] = {{in_data_raw[0][0]},
                                          {in_data_raw[1][0]},
                                          {in_data_raw[2][0]}};
      double * subdomain_in_data[NUM_SRC][1] = {{in_data_raw[0][1]},
                                                {in_data_raw[1][1]},
                                                {in_data_raw[2][1]}};
      double ** in_data[NUM_SRC][2] = {{remote_data[0], subdomain_in_data[0]},
                                       {remote_data[1], subdomain_in_data[1]},
                                       {remote_data[2], subdomain_in_data[2]}};
      double send_buffer[NUM_SRC][4] = {{-1,-1,-1,-1},{-1,-1,-1,-1},{-1,-1,-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_execute_interpolation_plan_element(interpolation_plan_element_stack[i],
                                                in_data[i], send_buffer[i]);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_[NUM_SRC];
      struct interpolation_method_data_request * data_request[NUM_SRC] =
         {&data_request_[0],&data_request_[1],&data_request_[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_get_source_exchange_data_interpolation_method(method_stack[i], &data_request[i], 1);

      int ref_requested_global_ids[6] = {2,5,8};
      int ref_requested_ranks[6] =      {2,2,2};

      struct interpolation_method_data_request ref_data_request[NUM_SRC] =
         {{0,NULL,NULL}, {3, ref_requested_global_ids,  ref_requested_ranks},
          {0,NULL,NULL}};

      unsigned ref_num_interpolated_points[NUM_SRC] = {2,2,0};
      int ref_global_ids[NUM_SRC][4] = {{0,2},{1,3},{-1}};
      double ref_out_data[NUM_SRC][4] = {{1,4+7},{2+3,5+6+8+9},{-1}};

      for (unsigned j = 0; j < NUM_SRC; ++j ) {

         check_data_request(data_request[j], &ref_data_request[j], 1);

         if (ref_num_interpolated_points[j] != num_interpolated_points[j][0])
            PUT_ERR( "3rd) wrong number of interpolated points\n");

         for (unsigned i = 0; i < ref_num_interpolated_points[j]; ++i) {
            if (ref_global_ids[j][i] != global_ids[j][i])
               PUT_ERR( "3rd) wrong global id order\n");
            // since the cells are on the sphere, we do not have perfect
            // rectangular cells and hence the reference data is very inaccurate...
            if ( fabs(send_buffer[j][i] - ref_out_data[j][i] ) > tol ) {
               PUT_ERR( "3rd) wrong result buffer\n");
               printf ( "3rd) results at (%u,%u) are %f %f \n", i, j,
                              send_buffer[j][i], ref_out_data[j][i] );
            }
         }
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned j = 0; j < NUM_SRC; ++j ) {
         for (unsigned i = 0; i < sizeof(method_stack[j])/sizeof(method_stack[j][0]); ++i)
            if (method_stack[j][i] != NULL)
               yac_delete_interpolation_method(method_stack[j][i]);
         for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack[j]) /
              sizeof(interpolation_plan_element_stack[j][0]); ++i) {
            yac_free_interpolation_plan_element(interpolation_plan_element_stack[j][i]);
            free(interpolation_plan_element_stack[j][i]);
         }
         delete_interpolation_method_data_request_stack(data_request[j], 1);
      }

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_points(&cell_points_src[i]);

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
         yac_global_search_delete(search[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_domain(&l_domain_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_delete_grid(grid_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_delete_grid(grid_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_named_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_named_tgt[i]);

      yac_reset_interpolation_method();
#undef NUM_SRC
#undef NUM_TGT
   }
#endif
#ifdef TEST4
   {  // 4th spmap recovery with two source and one target processes

      // the global source domain is a 4x3 grid:
      //
      //       15--27--16--28--17--29--18--30--19
      //       |       |       |       |       |
      //       19  08  21  09  23  10  25  11  26
      //       |       |       |       |       |
      //       10--18--11--20--12--22--13--24--14
      //       |       |       |       |       |
      //       10  04  12  05  14  06  16  07  17
      //       |       |       |       |       |
      //       05--09--06--11--07--13--08--15--09
      //       |       |       |       |       |
      //       01  00  03  01  05  02  07  03  08
      //       |       |       |       |       |
      //       00--00--01--02--02--04--03--06--04
      //
      // the global target domain is a 2x2 grid:
      //
      //       06--10--07--11--08
      //       |       |       |
      //       06  02  08  03  09
      //       |       |       |
      //       03--05--04--07--05
      //       |       |       |
      //       01  00  03  01  04
      //       |       |       |
      //       00--00--01--02--02
      //
      // target mask (X == masked):
      //
      //       +---+---+
      //       | O | X |
      //       +---+---+
      //       | O | X |
      //       +---+---+
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes

#define NUM_SRC 2
#define NUM_TGT 1

      struct communicator * comm_src[NUM_SRC], * comm_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_src[i] = yac_local_comm_new();

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_tgt[i] = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src[NUM_SRC], * comm_named_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_named_src[i] = yac_named_local_comm_new(comm_src[i], "src");

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_named_tgt[i] = yac_named_local_comm_new(comm_tgt[i], "tgt");

      // define domains on source

      struct grid * grid_src[NUM_SRC];

      unsigned num_cells_src[NUM_SRC][2] = {{4,3}, {2,3}};

      unsigned cyclic_src[2] = {0,0};

      double coordinates_x_src[NUM_SRC][5] = {{-1.5,-0.5,0.5,1.5,2.5},
                                              {0.5,1.5,2.5}};

      double coordinates_y_src[4] = {-1.5,-0.5,0.5,1.5};

      double coordinates_x_src_points[NUM_SRC][4] = {{-1.0*YAC_RAD, 0.0, 1.0*YAC_RAD, 2.0*YAC_RAD},
                                                     { 1.0*YAC_RAD, 2.0*YAC_RAD}};

      double coordinates_y_src_points[3] = {-1.0*YAC_RAD, 0.0, 1.0*YAC_RAD};

      for ( unsigned j = 0; j < NUM_SRC; ++j )
         for ( unsigned i = 0; i < 4; ++i )
            coordinates_x_src[j][i] *= YAC_RAD;

      for ( unsigned i = 0; i < 4; ++i )
         coordinates_y_src[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_SRC; ++i)
         grid_src[i] = yac_reg2d_grid_new(coordinates_x_src[i], coordinates_y_src,
               num_cells_src[i], cyclic_src);

      struct subdomain subdomain_src[NUM_SRC];

      int global_corner_ids_src[NUM_SRC][20] = {{0,1,2,3,4, 5,6,7,8,9, 10,11,12,13,14, 15,16,17,18,19},
                                                {2,3,4, 7,8,9, 12,13,14, 17,18,19}};

      unsigned corner_is_on_rank_src[NUM_SRC][20] = {{-1,-1,-1,1,1, -1,-1,-1,1,1, -1,-1,-1,1,1, -1,-1,-1,1,1},
                                                     {0,-1,-1, 0,-1,-1, 0,-1,-1, 0,-1,-1}};

      int global_cell_ids_src[NUM_SRC][12] = {{0,1,2,3, 4,5,6,7, 8,9,10,11},
                                              {2,3, 6,7, 10,11}};

      unsigned cell_is_on_rank_src[NUM_SRC][12] = {{-1,-1,-1,1, -1,-1,-1,1, -1,-1,-1,1},
                                                   {0,-1, 0,-1, 0,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_subdomain_unstruct(&subdomain_src[i], grid_src[i], comm_named_src[i],
                                    global_corner_ids_src[i], corner_is_on_rank_src[i],
                                    global_cell_ids_src[i], cell_is_on_rank_src[i],
                                    NULL, NULL);

      struct domain l_domain_src[NUM_SRC];

      struct subdomain * blocks_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i) blocks_src[i] = &subdomain_src[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_domain(&l_domain_src[i], comm_named_src[i], &blocks_src[i], 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt[NUM_TGT];
      unsigned num_cells_tgt[2] = {2,2};
      unsigned cyclic_tgt[2] = {0,0};
      double coordinates_x_tgt[3] = {-1,0,1};
      double coordinates_y_tgt[3] = {-1,0,1};

      double coordinates_x_tgt_point[2] = {-0.5,0.5};
      double coordinates_y_tgt_point[2] = {-0.5,0.5};

      for ( unsigned i = 0; i < 3; ++i ) {
        coordinates_x_tgt[i] *= YAC_RAD;
        coordinates_y_tgt[i] *= YAC_RAD;
      }

      for ( unsigned i = 0; i < 2; ++i ) {
        coordinates_x_tgt_point[i] *= YAC_RAD;
        coordinates_y_tgt_point[i] *= YAC_RAD;
      }

      for (unsigned i = 0; i < NUM_TGT; ++i)
        grid_tgt[i] = yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt,
                                     num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt[NUM_TGT];

      int global_corner_ids_tgt[9] = {0,1,2,3,4,5,6,7,8};
      unsigned corner_is_on_rank_tgt[NUM_TGT][9] = {{-1,-1,-1, -1,-1,-1, -1,-1,-1}};
      int global_cell_ids_tgt[4] = {0,1,2,3};
      unsigned cell_is_on_rank_tgt[NUM_TGT][4] = {{-1,-1,-1,-1}};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_subdomain_unstruct(&subdomain_tgt[i], grid_tgt[i], comm_named_tgt[i],
                                    global_corner_ids_tgt, corner_is_on_rank_tgt[i],
                                    global_cell_ids_tgt, cell_is_on_rank_tgt[i],
                                    NULL, NULL);

      struct domain l_domain_tgt[NUM_TGT];

      struct subdomain * blocks_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_TGT; ++i) blocks_tgt[i] = &subdomain_tgt[i];

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_domain(&l_domain_tgt[i], comm_named_tgt[i], &blocks_tgt[i], 1, "domain_tgt");

      // finish domain definition
      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_enddef_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_enddef_domain(&l_domain_tgt[i]);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[NUM_SRC + NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        search[i] =
          yac_global_search_new(&l_domain_src[i], "domain_tgt", comm_src[i],
                                1,0, "component_p1", "component_p2");
      for (unsigned i = 0; i < NUM_TGT; ++i)
        search[NUM_SRC + i] =
          yac_global_search_new(&l_domain_tgt[i], "domain_src", comm_tgt[i],
                                0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
        yac_enddef_global_search(search[i]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_init_points(&cell_points_src[i], grid_src[i], CELL,
                         coordinates_x_src_points[i], coordinates_y_src_points );

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain[NUM_SRC];
      struct points ** point_sets[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        points_per_subdomain[i] = &cell_points_src[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        point_sets[i] = &points_per_subdomain[i];

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        method_spmap[i] =
          yac_new_interpolation_method_spmap (search[i], &l_domain_src[i],
                                             "spmap stack", &point_sets[i], 1, 0 );

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[NUM_SRC][2];
      struct interpolation_plan_element * interpolation_plan_element_stack[NUM_SRC][2];

      for (unsigned i = 0; i < NUM_SRC; ++i) {
        method_stack[i][0] = method_spmap[i];
        method_stack[i][1] = NULL;
        interpolation_plan_element_stack[i][0] = NULL;
        interpolation_plan_element_stack[i][1] = NULL;
      }

      int global_ids[NUM_SRC][4] = {{0,2},{-1}};
      double coordinates_xyz[NUM_SRC][4][3];
      unsigned num_points[NUM_SRC] = {2,0};
      unsigned num_interpolated_points[NUM_SRC][1];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        for (unsigned j = 0; j < num_points[i]; ++j)
          LLtoXYZ(coordinates_x_tgt_point[global_ids[i][j]&1],
                  coordinates_y_tgt_point[global_ids[i][j]>>1],
                  coordinates_xyz[i][j]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_do_search_interpolation_method(method_stack[i], interpolation_plan_element_stack[i],
                                        global_ids[i], (num_points[i]>0)?coordinates_xyz[i]:NULL,
                                        CELL, num_points[i], num_interpolated_points[i], 1);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_enddef_interpolation_method(method_stack[i]);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[NUM_SRC][2][12] = {{{4,8,12},{1,2,3,-1, 5,6,7,-1, 9,10,11,-1}},
                                            {{-1},{-1,4, -1,8, -1,12}}};
      double * remote_data[NUM_SRC][1];
      double * subdomain_in_data[NUM_SRC][1];
      double ** in_data[NUM_SRC][2];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        remote_data[i][0] = in_data_raw[i][0];
      for (unsigned i = 0; i < NUM_SRC; ++i)
        subdomain_in_data[i][0] = in_data_raw[i][1];
      for (unsigned i = 0; i < NUM_SRC; ++i) {
        in_data[i][0] = remote_data[i];
        in_data[i][1] = subdomain_in_data[i];
      }

      double send_buffer[NUM_SRC][4] = {{-1,-1,-1,-1},{-1,-1,-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_execute_interpolation_plan_element(interpolation_plan_element_stack[i],
                                                in_data[i], send_buffer[i]);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_[NUM_SRC];
      struct interpolation_method_data_request * data_request[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i) data_request[i] = &data_request_[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_get_source_exchange_data_interpolation_method(
          method_stack[i], &data_request[i], 1);

      int ref_requested_global_ids[3] = {3,7,11};
      int ref_requested_ranks[3] = {1,1,1};
      struct interpolation_method_data_request ref_data_request[NUM_SRC] =
         {{3,ref_requested_global_ids,ref_requested_ranks}, {0,NULL,NULL}};

      unsigned ref_num_interpolated_points[NUM_SRC] = {2,0};
      int ref_global_ids[NUM_SRC][2] = {{0,2},{-1}};
      double ref_out_data[NUM_SRC][2] = {{10,68},{-1}};

      for (unsigned j = 0; j < NUM_SRC; ++j ) {

         check_data_request(data_request[j], &ref_data_request[j], 1);

         if (ref_num_interpolated_points[j] != num_interpolated_points[j][0])
            PUT_ERR( "4th) wrong number of interpolated points\n");

         for (unsigned i = 0; i < ref_num_interpolated_points[j]; ++i) {
            if (ref_global_ids[j][i] != global_ids[j][i])
               PUT_ERR( "4th) wrong global id order\n");
            // since the cells are on the sphere, we do not have perfect
            // rectangular cells and hence the reference data is very inaccurate...
            if ( fabs(send_buffer[j][i] - ref_out_data[j][i] ) > tol ) {
               PUT_ERR( "4th) wrong result buffer\n");
               printf ( "4th) results at (%u,%u) are %f %f \n",
                         i, j, send_buffer[j][i], ref_out_data[j][i] );
            }
         }
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned j = 0; j < NUM_SRC; ++j ) {
         for (unsigned i = 0; i < sizeof(method_stack[j])/sizeof(method_stack[j][0]); ++i)
            if (method_stack[j][i] != NULL)
               yac_delete_interpolation_method(method_stack[j][i]);
         for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack[j]) /
              sizeof(interpolation_plan_element_stack[j][0]); ++i) {
            yac_free_interpolation_plan_element(interpolation_plan_element_stack[j][i]);
            free(interpolation_plan_element_stack[j][i]);
         }
         delete_interpolation_method_data_request_stack(data_request[j], 1);
      }

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_points(&cell_points_src[i]);

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
         yac_global_search_delete(search[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_domain(&l_domain_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_delete_grid(grid_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_delete_grid(grid_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_named_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_named_tgt[i]);

      yac_reset_interpolation_method();
#undef NUM_SRC
#undef NUM_TGT
   }
#endif
#ifdef TEST5
   {  // 5th spmap recovery with two source and one target processes

      // the global source domain is a 4x3 grid:
      //
      //       15--27--16--28--17--29--18--30--19
      //       |       |       |       |       |
      //       19  08  21  09  23  10  25  11  26
      //       |       |       |       |       |
      //       10--18--11--20--12--22--13--24--14
      //       |       |       |       |       |
      //       10  04  12  05  14  06  16  07  17
      //       |       |       |       |       |
      //       05--09--06--11--07--13--08--15--09
      //       |       |       |       |       |
      //       01  00  03  01  05  02  07  03  08
      //       |       |       |       |       |
      //       00--00--01--02--02--04--03--06--04
      //
      // source mask (X == masked)
      //
      //       +---+---+---+---+
      //       | O | X | X | X |
      //       +---+---+---+---+
      //       | X | X | X | X |
      //       +---+---+---+---+
      //       | X | X | X | O |
      //       +---+---+---+---+
      //
      // the global target domain is a 4x2 grid:
      //
      //       08--14--09--15--10--16--11
      //       |       |       |       | 
      //       08  03  10  04  12  05  13
      //       |       |       |       | 
      //       04--07--05--09--06--11--07
      //       |       |       |       | 
      //       01  00  03  01  05  02  06
      //       |       |       |       | 
      //       00--00--01--02--02--04--03
      //
      // target mask (X == masked):
      //
      //       +---+---+---+
      //       | X | X | O |
      //       +---+---+---+
      //       | X | X | O |
      //       +---+---+---+
      //
      // Both source processes have one non-masked source point, but
      // only one source process has the two non-masked target points.
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes

#define NUM_SRC 2
#define NUM_TGT 1

      struct communicator * comm_src[NUM_SRC], * comm_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_src[i] = yac_local_comm_new();

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_tgt[i] = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src[NUM_SRC], * comm_named_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_named_src[i] = yac_named_local_comm_new(comm_src[i], "src");

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_named_tgt[i] = yac_named_local_comm_new(comm_tgt[i], "tgt");

      // define domains on source

      struct grid * grid_src[NUM_SRC];

      unsigned num_cells_src[2] = {3,3};

      unsigned cyclic_src[2] = {0,0};

      double coordinates_x_src[NUM_SRC][4] = {{-2,-1, 0, 1},
                                              {-1, 0, 1, 2}};

      double coordinates_y_src[4] = {-1.5,-0.5,0.5,1.5};

      double coordinates_x_src_points[NUM_SRC][3] = {{-1.5,-0.5, 0.5},
                                                     {-0.5, 0.5, 1.5}};

      double coordinates_y_src_points[3] = {-1.0, 0.0, 1.0};

      unsigned cell_mask_values_src[NUM_SRC][9] = {{1,1,1, 1,1,1, 0,1,1},
                                                   {1,1,0, 1,1,1, 1,1,1}};

      for ( unsigned j = 0; j < NUM_SRC; ++j )
        for ( unsigned i = 0; i < 4; ++i )
          coordinates_x_src[j][i] *= YAC_RAD;

      for ( unsigned i = 0; i < 4; ++i )
         coordinates_y_src[i] *= YAC_RAD;

      for ( unsigned j = 0; j < NUM_SRC; ++j )
        for ( unsigned i = 0; i < 3; ++i )
          coordinates_x_src_points[j][i] *= YAC_RAD;

      for ( unsigned i = 0; i < 3; ++i )
         coordinates_y_src_points[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_SRC; ++i)
         grid_src[i] = yac_reg2d_grid_new(coordinates_x_src[i], coordinates_y_src,
               num_cells_src, cyclic_src);

      struct subdomain subdomain_src[NUM_SRC];

      int global_corner_ids_src[NUM_SRC][16] = {{0,1,2,3, 5,6,7,8, 10,11,12,13, 15,16,17,18},
                                                {1,2,3,4, 6,7,8,9, 11,12,13,14, 16,17,18,19}};

      unsigned corner_is_on_rank_src[NUM_SRC][16] = {{-1,-1,-1,1, -1,-1,-1,1, -1,-1,1,1, -1,-1,1,1},
                                                     {0,0,-1,-1, 0,0,-1,-1, 0,-1,-1,-1, 0,-1,-1,-1}};

      int global_cell_ids_src[NUM_SRC][9] = {{0,1,2, 4,5,6, 8,9,10},
                                             {1,2,3, 5,6,7, 9,10,11}};

      unsigned cell_is_on_rank_src[NUM_SRC][9] = {{-1,-1,1, -1,-1,1, -1,-1,1},
                                                  {0,-1,-1, 0,-1,-1, 0,-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_subdomain_unstruct(&subdomain_src[i], grid_src[i], comm_named_src[i],
                                    global_corner_ids_src[i], corner_is_on_rank_src[i],
                                    global_cell_ids_src[i], cell_is_on_rank_src[i],
                                    NULL, NULL);

      struct domain l_domain_src[NUM_SRC];

      struct subdomain * blocks_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i) blocks_src[i] = &subdomain_src[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_domain(&l_domain_src[i], comm_named_src[i], &blocks_src[i], 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt[NUM_TGT];
      unsigned num_cells_tgt[2] = {3,2};
      unsigned cyclic_tgt[2] = {0,0};
      double coordinates_x_tgt[4] = {-1.5,-0.5,0.5,1.5};
      double coordinates_y_tgt[3] = {-1,0,1};

      for ( unsigned i = 0; i < 4; ++i )
        coordinates_x_tgt[i] *= YAC_RAD;

      for ( unsigned i = 0; i < 3; ++i )
        coordinates_y_tgt[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_TGT; ++i)
        grid_tgt[i] = yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt,
                                         num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt;

      int global_corner_ids_tgt[12] = {0,1,2,3, 4,5,6,7, 8,9,10,11};
      unsigned corner_is_on_rank_tgt[12] = {-1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1};
      int global_cell_ids_tgt[6] = {0,1,2,3,4,5};
      unsigned cell_is_on_rank_tgt[6] = {-1,-1,-1,-1,-1,-1};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_subdomain_unstruct(&subdomain_tgt, grid_tgt[i], comm_named_tgt[i],
                                    global_corner_ids_tgt, corner_is_on_rank_tgt,
                                    global_cell_ids_tgt, cell_is_on_rank_tgt,
                                    NULL, NULL);

      struct domain l_domain_tgt;

      struct subdomain * blocks_tgt;

      for (unsigned i = 0; i < NUM_TGT; ++i) blocks_tgt = &subdomain_tgt;

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_domain(&l_domain_tgt, comm_named_tgt[i], &blocks_tgt, 1, "domain_tgt");

      // finish domain definition
      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_enddef_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_enddef_domain(&l_domain_tgt);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[NUM_SRC + NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        search[i] =
          yac_global_search_new(&l_domain_src[i], "domain_tgt", comm_src[i],
                                1,0, "component_p1", "component_p2");
      for (unsigned i = 0; i < NUM_TGT; ++i)
        search[NUM_SRC + i] =
          yac_global_search_new(&l_domain_tgt, "domain_src", comm_tgt[i],
                                0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
        yac_enddef_global_search(search[i]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_init_points(&cell_points_src[i], grid_src[i], CELL,
                         coordinates_x_src_points[i],
                         coordinates_y_src_points );

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain[NUM_SRC];
      struct points ** point_sets[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        points_per_subdomain[i] = &cell_points_src[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        point_sets[i] = &points_per_subdomain[i];

      //-------------------------------
      // initialise mask on source side
      //-------------------------------

      struct mask cell_mask_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_mask(&cell_mask_src[i], cell_mask_values_src[i],
                      &cell_points_src[i]);

      //--------------------------------
      // set up mask sets on source side
      //--------------------------------

      struct mask * masks_per_subdomain[NUM_SRC];
      struct mask ** mask_sets[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        masks_per_subdomain[i] = &cell_mask_src[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        mask_sets[i] = &masks_per_subdomain[i];

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        method_spmap[i] =
          yac_new_interpolation_method_spmap_mask(
            search[i], &l_domain_src[i], "spmap stack", &point_sets[i],
            &mask_sets[i], 1, 0, 1 );

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[NUM_SRC][2];
      struct interpolation_plan_element * interpolation_plan_element_stack[NUM_SRC][2];

      for (unsigned i = 0; i < NUM_SRC; ++i) {
        method_stack[i][0] = method_spmap[i];
        method_stack[i][1] = NULL;
        interpolation_plan_element_stack[i][0] = NULL;
        interpolation_plan_element_stack[i][1] = NULL;
      }

      int global_ids[NUM_SRC][4] = {{-1},{2,5}};
      double coordinates_xyz[NUM_SRC][2][3];
      unsigned num_points[NUM_SRC] = {0,2};
      unsigned num_interpolated_points[NUM_SRC][1];
      LLtoXYZ_deg(0.0,  0.0, coordinates_xyz[1][0]); // dummy
      LLtoXYZ_deg(0.0,  0.0, coordinates_xyz[1][0]); // dummy
      LLtoXYZ_deg(1.0, -0.5, coordinates_xyz[1][0]);
      LLtoXYZ_deg(1.0,  0.5, coordinates_xyz[1][1]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_do_search_interpolation_method(
            method_stack[i], interpolation_plan_element_stack[i], global_ids[i],
            (num_points[i]>0)?coordinates_xyz[i]:NULL, CELL, num_points[i],
            num_interpolated_points[i], 1);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_enddef_interpolation_method(method_stack[i]);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[NUM_SRC][2][9] = {{{-1},{1,2,-1, 5,6,-1, 9,10,-1}},
                                           {{9},{-1,3,4, -1,7,8, -1,11,12}}};
      double * remote_data[NUM_SRC][1];
      double * subdomain_in_data[NUM_SRC][1];
      double ** in_data[NUM_SRC][2];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        remote_data[i][0] = in_data_raw[i][0];
      for (unsigned i = 0; i < NUM_SRC; ++i)
        subdomain_in_data[i][0] = in_data_raw[i][1];
      for (unsigned i = 0; i < NUM_SRC; ++i) {
        in_data[i][0] = remote_data[i];
        in_data[i][1] = subdomain_in_data[i];
      }

      double send_buffer[NUM_SRC][2] = {{-1,-1},{-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_execute_interpolation_plan_element(interpolation_plan_element_stack[i],
                                                in_data[i], send_buffer[i]);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_[NUM_SRC];
      struct interpolation_method_data_request * data_request[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i) data_request[i] = &data_request_[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_get_source_exchange_data_interpolation_method(method_stack[i], &data_request[i], 1);

      struct interpolation_method_data_request ref_data_request[NUM_SRC] =
         {{0,NULL,NULL}, {1,(int[]){8},(int[]){0}}};

      unsigned ref_num_interpolated_points[NUM_SRC] = {0,2};
      int ref_global_ids[NUM_SRC][2] = {{-1},{2,5}};
      double ref_out_data[NUM_SRC][2] = {{-1},{4,9}};

      for (unsigned j = 0; j < NUM_SRC; ++j ) {

         check_data_request(data_request[j], &ref_data_request[j], 1);

         if (ref_num_interpolated_points[j] != num_interpolated_points[j][0]) {
            printf("5th) wrong number of interpolated points on SRC %u: %u %u\n",
                   j, ref_num_interpolated_points[j], num_interpolated_points[j][0] );
            PUT_ERR("5th) wrong number of interpolated points\n");
         }

         for (unsigned i = 0; i < ref_num_interpolated_points[j]; ++i) {
            if (ref_global_ids[j][i] != global_ids[j][i]) {
               printf ( "5th) wrong global ID order at (%u,%u) are %d %d \n",
                        j, i, ref_global_ids[j][i], global_ids[j][i] );
               PUT_ERR( "5th) wrong global ID order\n");
            }
            // since the cells are on the sphere, we do not have perfect
            // rectangular cells and hence the reference data is very inaccurate...
            if ( fabs(send_buffer[j][i] - ref_out_data[j][i] ) > tol ) {
               printf ( "5th) results for ID %d at (%u,%u) are %f %f \n",
                        ref_global_ids[j][i], i, j, send_buffer[j][i], ref_out_data[j][i] );
               PUT_ERR( "5th) wrong result buffer\n");
            }
         }
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned j = 0; j < NUM_SRC; ++j ) {
         for (unsigned i = 0; i < sizeof(method_stack[j])/sizeof(method_stack[j][0]); ++i)
            if (method_stack[j][i] != NULL)
               yac_delete_interpolation_method(method_stack[j][i]);
         for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack[j]) /
              sizeof(interpolation_plan_element_stack[j][0]); ++i) {
            yac_free_interpolation_plan_element(interpolation_plan_element_stack[j][i]);
            free(interpolation_plan_element_stack[j][i]);
         }
         delete_interpolation_method_data_request_stack(data_request[j], 1);
      }

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_mask(&cell_mask_src[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_points(&cell_points_src[i]);

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
         yac_global_search_delete(search[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_domain(&l_domain_tgt);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_delete_grid(grid_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_delete_grid(grid_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_named_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_named_tgt[i]);

      yac_reset_interpolation_method();
#undef NUM_SRC
#undef NUM_TGT
   }
#endif
#ifdef TEST6
   {  // 3rd spmap recovery with three source and two target processes

      // the global source domain is a 4x3 grid:
      //
      //       15--27--16--28--17--29--18--30--19
      //       |       |       |       |       |
      //       19  08  21  09  23  10  25  11  26
      //       |       |       |       |       |
      //       10--18--11--20--12--22--13--24--14
      //       |       |       |       |       |
      //       10  04  12  05  14  06  16  07  17
      //       |       |       |       |       |
      //       05--09--06--11--07--13--08--15--09
      //       |       |       |       |       |
      //       01  00  03  01  05  02  07  03  08
      //       |       |       |       |       |
      //       00--00--01--02--02--04--03--06--04
      //
      // the global target domain is a 3x2 grid:
      //
      //        08--14--09--15--10--16--11
      //        |       |       |       |
      //        08  03  10  04  12  05  13
      //        |       |       |       |
      //        04--07--05--09--06--11--07
      //        |       |       |       |
      //        01  00  03  01  05  02  06
      //        |       |       |       |
      //        00--00--01--02--02--04--03
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes

      #define NUM_SRC 3
      #define NUM_TGT 2

      struct communicator * comm_src[NUM_SRC], * comm_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_src[i] = yac_local_comm_new();

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_tgt[i] = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src[NUM_SRC], * comm_named_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_named_src[i] = yac_named_local_comm_new(comm_src[i], "src");

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_named_tgt[i] = yac_named_local_comm_new(comm_tgt[i], "tgt");

      // define domains on source

      struct grid * grid_src[NUM_SRC];

      unsigned num_cells_src[NUM_SRC][2] = {{2,3}, {3,3}, {3,3}};

      unsigned cyclic_src[2] = {0,0};

      double coordinates_x_src[NUM_SRC][4] = {{-1.5,-0.5,0.5},
                                              {-1.5,-0.5,0.5,1.5},
                                              {-0.5,0.5,1.5,2.5}};
      double coordinates_y_src[4] = {-1.5,-0.5,0.5,1.5};

      double coordinates_x_src_points[NUM_SRC][3] = {{-1.0*YAC_RAD, 0.0},
                                                     {-1.0*YAC_RAD, 0.0, 1.0*YAC_RAD},
                                                     { 0.0, 1.0*YAC_RAD, 2.0*YAC_RAD}};

      double coordinates_y_src_points[3] = {-1.0*YAC_RAD, 0.0, 1.0*YAC_RAD};

      for ( unsigned j = 0; j < NUM_SRC; ++j )
         for ( unsigned i = 0; i < 4; ++i )
            coordinates_x_src[j][i] *= YAC_RAD;

      for ( unsigned i = 0; i < 4; ++i )
         coordinates_y_src[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_SRC; ++i)
         grid_src[i] =
            yac_reg2d_grid_new(coordinates_x_src[i], coordinates_y_src,
                               num_cells_src[i], cyclic_src);

      struct subdomain subdomain_src[NUM_SRC];

      int global_corner_ids_src[NUM_SRC][16] = {{0,1,2, 5,6,7, 10,11,12, 15,16,17},
                                                {0,1,2,3, 5,6,7,8, 10,11,12,13, 15,16,17,18},
                                                {1,2,3,4, 6,7,8,9, 11,12,13,14, 16,17,18,19}};

      unsigned corner_is_on_rank_src[NUM_SRC][16] = {{-1,-1,1, -1,-1,1, -1,-1,1, -1,-1,1},
                                                     {0,-1,-1,2, 0,-1,-1,2, 0,-1,-1,2, 0,-1,-1,2},
                                                     {1,-1,-1,-1, 1,-1,-1,-1, 1,-1,-1,-1, 1,-1,-1,-1}};

      int global_cell_ids_src[NUM_SRC][9] = {{0,1,4,5,8,9},
                                             {0,1,2,4,5,6,8,9,10},
                                             {1,2,3,5,6,7,9,10,11}};

      unsigned cell_is_on_rank_src[NUM_SRC][9] = {{-1,1,-1,1,-1,1},
                                                  {0,-1,2,0,-1,2,0,-1,2},
                                                  {1,-1,-1,1,-1,-1,1,-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_subdomain_unstruct(&subdomain_src[i], grid_src[i], comm_named_src[i],
                                    global_corner_ids_src[i], corner_is_on_rank_src[i],
                                    global_cell_ids_src[i], cell_is_on_rank_src[i],
                                    NULL, NULL);

      struct domain l_domain_src[NUM_SRC];

      struct subdomain * blocks_src[NUM_SRC] = {&subdomain_src[0],
                                                &subdomain_src[1],
                                                &subdomain_src[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_domain(
          &l_domain_src[i], comm_named_src[i], &blocks_src[i], 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt[NUM_TGT];
      unsigned num_cells_tgt[2] = {3,2};
      unsigned cyclic_tgt[2] = {0,0};
      double coordinates_x_tgt[4] = {-1,0,1,2};
      double coordinates_y_tgt[3] = {-1,0,1};

      double coordinates_x_tgt_point[3] = {-0.75,0.5,1.75};
      double coordinates_y_tgt_point[2] = {-0.75,0.5};

      for ( unsigned i = 0; i < 4; ++i ) coordinates_x_tgt[i] *= YAC_RAD;
      for ( unsigned i = 0; i < 3; ++i ) coordinates_y_tgt[i] *= YAC_RAD;

      for ( unsigned i = 0; i < 3; ++i ) coordinates_x_tgt_point[i] *= YAC_RAD;
      for ( unsigned i = 0; i < 2; ++i ) coordinates_y_tgt_point[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_TGT; ++i)
        grid_tgt[i] =
          yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt,
                             num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt[NUM_TGT];

      int global_corner_ids_tgt[12] = {0,1,2,3,4,5,6,7,8,9,10,11};
      unsigned corner_is_on_rank_tgt[NUM_TGT][12] =
        {{-1,-1,-1,-1, -1,-1,-1,-1, 1,1,1,1},
         {0,0,0,0,     -1,-1,-1,-1, -1,-1,-1,-1}};
      int global_cell_ids_tgt[6] = {0,1,2,3,4,5};
      unsigned cell_is_on_rank_tgt[NUM_TGT][6] =
        {{-1,-1,-1,1,1,1},{0,0,0,-1,-1,-1}};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_subdomain_unstruct(
          &subdomain_tgt[i], grid_tgt[i], comm_named_tgt[i],
          global_corner_ids_tgt, corner_is_on_rank_tgt[i],
          global_cell_ids_tgt, cell_is_on_rank_tgt[i], NULL, NULL);

      struct domain l_domain_tgt[NUM_TGT];

      struct subdomain * blocks_tgt[NUM_TGT] = {&subdomain_tgt[0],
                                                &subdomain_tgt[1]};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_domain(
          &l_domain_tgt[i], comm_named_tgt[i], &blocks_tgt[i], 1, "domain_tgt");

      // finish domain definition
      for (unsigned i = 0; i < NUM_SRC; ++i) yac_enddef_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i) yac_enddef_domain(&l_domain_tgt[i]);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[NUM_SRC + NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        search[i] =
          yac_global_search_new(&l_domain_src[i], "domain_tgt", comm_src[i],
                                1,0, "component_p1", "component_p2");
      for (unsigned i = 0; i < NUM_TGT; ++i)
        search[NUM_SRC + i] =
          yac_global_search_new(&l_domain_tgt[i], "domain_src", comm_tgt[i],
                                0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
        yac_enddef_global_search(search[i]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_init_points(&cell_points_src[i], grid_src[i], CELL,
                         coordinates_x_src_points[i], coordinates_y_src_points );

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain[NUM_SRC] = {&cell_points_src[0],
                                                       &cell_points_src[1],
                                                       &cell_points_src[2]};
      struct points ** point_sets[NUM_SRC] = {&points_per_subdomain[0],
                                              &points_per_subdomain[1],
                                              &points_per_subdomain[2]};

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        method_spmap[i] =
          yac_new_interpolation_method_spmap(
            search[i], &l_domain_src[i], "spmap stack", &point_sets[i], 1, 0);

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[NUM_SRC][2] =
        {{method_spmap[0], NULL},
         {method_spmap[1], NULL},
         {method_spmap[2], NULL}};

      struct interpolation_plan_element
        * interpolation_plan_element_stack[NUM_SRC][2] =
          {{NULL, NULL},{NULL, NULL},{NULL, NULL}};
      int global_ids[NUM_SRC][6] = {{0,1,2,3,4,5},{0,1,2,3,4,5},{0,1,2,3,4,5}};
      double coordinates_xyz[NUM_SRC][6][3];
      unsigned num_points = 6;
      unsigned num_interpolated_points[NUM_SRC][1];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        for (unsigned j = 0; j < num_points; ++j)
          LLtoXYZ(coordinates_x_tgt_point[j%3],
                  coordinates_y_tgt_point[j/3],
                  coordinates_xyz[i][j]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_do_search_interpolation_method(
            method_stack[i], interpolation_plan_element_stack[i], global_ids[i],
            coordinates_xyz[i], CELL, num_points, num_interpolated_points[i], 1);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_enddef_interpolation_method(method_stack[i]);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[NUM_SRC][2][9] = {{{-1},{1,-1,5,-1,9,-1}},
                                           {{3,7,11},{-1,2,-1,-1,6,-1,-1,10,-1}},
                                           {{-1},{-1,3,4,-1,7,8,-1,11,12}}};
      double * remote_data[NUM_SRC][1] = {{in_data_raw[0][0]},
                                          {in_data_raw[1][0]},
                                          {in_data_raw[2][0]}};
      double * subdomain_in_data[NUM_SRC][1] = {{in_data_raw[0][1]},
                                                {in_data_raw[1][1]},
                                                {in_data_raw[2][1]}};
      double ** in_data[NUM_SRC][2] = {{remote_data[0], subdomain_in_data[0]},
                                       {remote_data[1], subdomain_in_data[1]},
                                       {remote_data[2], subdomain_in_data[2]}};
      double send_buffer[NUM_SRC][4] = {{-1,-1,-1,-1},{-1,-1,-1,-1},{-1,-1,-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_execute_interpolation_plan_element(interpolation_plan_element_stack[i],
                                                in_data[i], send_buffer[i]);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_[NUM_SRC];
      struct interpolation_method_data_request * data_request[NUM_SRC] =
         {&data_request_[0],&data_request_[1],&data_request_[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_get_source_exchange_data_interpolation_method(method_stack[i], &data_request[i], 1);

      int ref_requested_global_ids[6] = {2,6,10};
      int ref_requested_ranks[6] =      {2,2,2};

      struct interpolation_method_data_request ref_data_request[NUM_SRC] =
         {{0,NULL,NULL}, {3, ref_requested_global_ids,  ref_requested_ranks},
          {0,NULL,NULL}};

      unsigned ref_num_interpolated_points[NUM_SRC] = {2,2,2};
      int ref_global_ids[NUM_SRC][4] = {{0,3},{1,4},{2,5}};
      double ref_out_data[NUM_SRC][4] = {{1,5+9},{2+3,6+7+10+11},{4,8+12}};

      for (unsigned j = 0; j < NUM_SRC; ++j ) {

         check_data_request(data_request[j], &ref_data_request[j], 1);

         if (ref_num_interpolated_points[j] != num_interpolated_points[j][0])
            PUT_ERR( "3rd) wrong number of interpolated points\n");

         for (unsigned i = 0; i < ref_num_interpolated_points[j]; ++i) {
            if (ref_global_ids[j][i] != global_ids[j][i])
               PUT_ERR( "3rd) wrong global id order\n");
            // since the cells are on the sphere, we do not have perfect
            // rectangular cells and hence the reference data is very inaccurate...
            if ( fabs(send_buffer[j][i] - ref_out_data[j][i] ) > tol ) {
               PUT_ERR( "3rd) wrong result buffer\n");
               printf ( "3rd) results at (%u,%u) are %f %f \n", i, j,
                              send_buffer[j][i], ref_out_data[j][i] );
            }
         }
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned j = 0; j < NUM_SRC; ++j ) {
         for (unsigned i = 0; i < sizeof(method_stack[j])/sizeof(method_stack[j][0]); ++i)
            if (method_stack[j][i] != NULL)
               yac_delete_interpolation_method(method_stack[j][i]);
         for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack[j]) /
              sizeof(interpolation_plan_element_stack[j][0]); ++i) {
            yac_free_interpolation_plan_element(interpolation_plan_element_stack[j][i]);
            free(interpolation_plan_element_stack[j][i]);
         }
         delete_interpolation_method_data_request_stack(data_request[j], 1);
      }

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_points(&cell_points_src[i]);

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
         yac_global_search_delete(search[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_domain(&l_domain_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_delete_grid(grid_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_delete_grid(grid_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_named_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_named_tgt[i]);

      yac_reset_interpolation_method();
#undef NUM_SRC
#undef NUM_TGT
   }
#endif
#ifdef TEST7
   {  // spmap recovery with three source and two target processes
      // checks request forwarding

      // the global source domain is a 3x2 grid:
      //
      //       08--13--09--14--10--15--11
      //       |       |       |       |
      //       07  03  09  04  11  05  12
      //       |       |       |       |
      //       04--06--05--08--06--10--07
      //       |       |       |       |
      //       01  00  03  01  05  02  06
      //       |       |       |       |
      //       00--00--01--02--02--04--03
      //
      // the global target domain is a 1x2 grid:
      //
      //        04--06--05
      //        |       |
      //        04  01  05
      //        |       |
      //        02--03--03
      //        |       |
      //        01  00  02
      //        |       |
      //        00--00--01
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes

      #define NUM_SRC 3
      #define NUM_TGT 2

      struct communicator * comm_src[NUM_SRC], * comm_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_src[i] = yac_local_comm_new();

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_tgt[i] = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src[NUM_SRC], * comm_named_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_named_src[i] = yac_named_local_comm_new(comm_src[i], "src");

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_named_tgt[i] = yac_named_local_comm_new(comm_tgt[i], "tgt");

      // define domains on source

      struct grid * grid_src[NUM_SRC];

      unsigned num_cells_src[NUM_SRC][2] = {{2,2}, {2,2}, {2,2}};

      unsigned cyclic_src[2] = {0,0};

      double coordinates_x_src[NUM_SRC][3] = {{0.0,1.0,2.0},
                                              {1.0,2.0,3.0},
                                              {1.0,2.0,3.0}};
      double coordinates_y_src[3] = {0.0,1.0,2.0};

      double coordinates_x_src_points[NUM_SRC][2] = {{0.5*YAC_RAD, 1.5*YAC_RAD},
                                                     {1.5*YAC_RAD, 2.5*YAC_RAD},
                                                     {1.5*YAC_RAD, 2.5*YAC_RAD}};

      double coordinates_y_src_points[2] = {0.5*YAC_RAD, 1.5*YAC_RAD};

      unsigned cell_mask_values_src[NUM_SRC][4] = {{1,1,1,1},
                                                   {1,1,1,0},
                                                   {1,0,1,1}};

      for ( unsigned j = 0; j < NUM_SRC; ++j )
         for ( unsigned i = 0; i < 3; ++i )
            coordinates_x_src[j][i] *= YAC_RAD;

      for ( unsigned i = 0; i < 3; ++i )
         coordinates_y_src[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_SRC; ++i)
         grid_src[i] =
            yac_reg2d_grid_new(coordinates_x_src[i], coordinates_y_src,
                               num_cells_src[i], cyclic_src);

      struct subdomain subdomain_src[NUM_SRC];

      int global_corner_ids_src[NUM_SRC][9] = {{0,1,2, 4,5,6, 8,9,10},
                                               {1,2,3, 5,6,7, 9,10,11},
                                               {1,2,3, 5,6,7, 9,10,11}};

      unsigned corner_is_on_rank_src[NUM_SRC][9] = {{-1,-1,2, -1,-1,1, -1,-1,1},
                                                    {0,2,2, 0,-1,-1, 0,-1,-1},
                                                    {0,-1,-1, 0,1,1, 0,1,1}};

      int global_cell_ids_src[NUM_SRC][4] = {{0,1,3,4},
                                             {1,2,4,5},
                                             {1,2,4,5}};

      unsigned cell_is_on_rank_src[NUM_SRC][4] = {{-1,2-1,1},
                                                  {0,-1,0,-1},
                                                  {0,-1,0,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_subdomain_unstruct(&subdomain_src[i], grid_src[i], comm_named_src[i],
                                    global_corner_ids_src[i], corner_is_on_rank_src[i],
                                    global_cell_ids_src[i], cell_is_on_rank_src[i],
                                    NULL, NULL);

      struct domain l_domain_src[NUM_SRC];

      struct subdomain * blocks_src[NUM_SRC] = {&subdomain_src[0],
                                                &subdomain_src[1],
                                                &subdomain_src[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_domain(
          &l_domain_src[i], comm_named_src[i], &blocks_src[i], 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt[NUM_TGT];
      unsigned num_cells_tgt[2] = {1,2};
      unsigned cyclic_tgt[2] = {0,0};
      double coordinates_x_tgt[2] = {2.0,3.0};
      double coordinates_y_tgt[3] = {0.0,1.0,2.0};

      double coordinates_x_tgt_point[1] = {2.5};
      double coordinates_y_tgt_point[2] = {0.5,1.5};

      for ( unsigned i = 0; i < 2; ++i ) coordinates_x_tgt[i] *= YAC_RAD;
      for ( unsigned i = 0; i < 3; ++i ) coordinates_y_tgt[i] *= YAC_RAD;

      for ( unsigned i = 0; i < 1; ++i ) coordinates_x_tgt_point[i] *= YAC_RAD;
      for ( unsigned i = 0; i < 2; ++i ) coordinates_y_tgt_point[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_TGT; ++i)
        grid_tgt[i] =
          yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt,
                             num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt[NUM_TGT];

      int global_corner_ids_tgt[6] = {0,1,2,3,4,5};
      unsigned corner_is_on_rank_tgt[NUM_TGT][6] =
        {{-1,-1,-1,-1,1,1}, {0,0,0,0,-1,-1}};
      int global_cell_ids_tgt[2] = {0,1};
      unsigned cell_is_on_rank_tgt[NUM_TGT][2] = {{-1,1},{0,-1}};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_subdomain_unstruct(
          &subdomain_tgt[i], grid_tgt[i], comm_named_tgt[i],
          global_corner_ids_tgt, corner_is_on_rank_tgt[i],
          global_cell_ids_tgt, cell_is_on_rank_tgt[i], NULL, NULL);

      struct domain l_domain_tgt[NUM_TGT];

      struct subdomain * blocks_tgt[NUM_TGT] = {&subdomain_tgt[0],
                                                &subdomain_tgt[1]};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_domain(
          &l_domain_tgt[i], comm_named_tgt[i], &blocks_tgt[i], 1, "domain_tgt");

      // finish domain definition
      for (unsigned i = 0; i < NUM_SRC; ++i) yac_enddef_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i) yac_enddef_domain(&l_domain_tgt[i]);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[NUM_SRC + NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        search[i] =
          yac_global_search_new(&l_domain_src[i], "domain_tgt", comm_src[i],
                                1,0, "component_p1", "component_p2");
      for (unsigned i = 0; i < NUM_TGT; ++i)
        search[NUM_SRC + i] =
          yac_global_search_new(&l_domain_tgt[i], "domain_src", comm_tgt[i],
                                0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
        yac_enddef_global_search(search[i]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_init_points(&cell_points_src[i], grid_src[i], CELL,
                         coordinates_x_src_points[i], coordinates_y_src_points );

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain[NUM_SRC] = {&cell_points_src[0],
                                                       &cell_points_src[1],
                                                       &cell_points_src[2]};
      struct points ** point_sets[NUM_SRC] = {&points_per_subdomain[0],
                                              &points_per_subdomain[1],
                                              &points_per_subdomain[2]};

      //-------------------------------
      // initialise mask on source side
      //-------------------------------

      struct mask cell_mask_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_mask(&cell_mask_src[i], cell_mask_values_src[i],
                      &cell_points_src[i]);

      //--------------------------------
      // set up mask sets on source side
      //--------------------------------

      struct mask * masks_per_subdomain[NUM_SRC];
      struct mask ** mask_sets[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        masks_per_subdomain[i] = &cell_mask_src[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        mask_sets[i] = &masks_per_subdomain[i];

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        method_spmap[i] =
          yac_new_interpolation_method_spmap_mask(
            search[i], &l_domain_src[i], "spmap stack", &point_sets[i],
            &mask_sets[i], 1, 0, 1);

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[NUM_SRC][2] =
        {{method_spmap[0], NULL},
         {method_spmap[1], NULL},
         {method_spmap[2], NULL}};

      struct interpolation_plan_element
        * interpolation_plan_element_stack[NUM_SRC][2] =
          {{NULL, NULL},{NULL, NULL},{NULL, NULL}};
      int global_ids[NUM_SRC][2] = {{0,1},{0,1},{0,1}};
      double coordinates_xyz[NUM_SRC][2][3];
      unsigned num_points = 2;
      unsigned num_interpolated_points[NUM_SRC][1];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        for (unsigned j = 0; j < num_points; ++j)
          LLtoXYZ(coordinates_x_tgt_point[0],
                  coordinates_y_tgt_point[j],
                  coordinates_xyz[i][j]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_do_search_interpolation_method(
            method_stack[i], interpolation_plan_element_stack[i], global_ids[i],
            coordinates_xyz[i], CELL, num_points, num_interpolated_points[i], 1);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_enddef_interpolation_method(method_stack[i]);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[NUM_SRC][2][4] = {{{-1},{-1,-1,-1,-1}},
                                           {{-1},{-1,-1,-1,5}},
                                           {{-1},{-1,2,-1,-1}}};
      double * remote_data[NUM_SRC][1] = {{in_data_raw[0][0]},
                                          {in_data_raw[1][0]},
                                          {in_data_raw[2][0]}};
      double * subdomain_in_data[NUM_SRC][1] = {{in_data_raw[0][1]},
                                                {in_data_raw[1][1]},
                                                {in_data_raw[2][1]}};
      double ** in_data[NUM_SRC][2] = {{remote_data[0], subdomain_in_data[0]},
                                       {remote_data[1], subdomain_in_data[1]},
                                       {remote_data[2], subdomain_in_data[2]}};
      double send_buffer[NUM_SRC][2] = {{-1,-1},{-1,-1},{-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_execute_interpolation_plan_element(interpolation_plan_element_stack[i],
                                                in_data[i], send_buffer[i]);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_[NUM_SRC];
      struct interpolation_method_data_request * data_request[NUM_SRC] =
         {&data_request_[0],&data_request_[1],&data_request_[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_get_source_exchange_data_interpolation_method(method_stack[i], &data_request[i], 1);

      struct interpolation_method_data_request ref_data_request[NUM_SRC] =
         {{0,NULL,NULL}, {0,NULL,NULL}, {0,NULL,NULL}};

      unsigned ref_num_interpolated_points[NUM_SRC] = {0,1,1};
      int ref_global_ids[NUM_SRC][2] = {{-1},{1},{0}};
      double ref_out_data[NUM_SRC][2] = {{-1},{5},{2}};

      for (unsigned j = 0; j < NUM_SRC; ++j ) {

         check_data_request(data_request[j], &ref_data_request[j], 1);

         if (ref_num_interpolated_points[j] != num_interpolated_points[j][0])
            PUT_ERR( "3rd) wrong number of interpolated points\n");

         for (unsigned i = 0; i < ref_num_interpolated_points[j]; ++i) {
            if (ref_global_ids[j][i] != global_ids[j][i])
               PUT_ERR( "3rd) wrong global id order\n");
            // since the cells are on the sphere, we do not have perfect
            // rectangular cells and hence the reference data is very inaccurate...
            if ( fabs(send_buffer[j][i] - ref_out_data[j][i] ) > tol ) {
               PUT_ERR( "3rd) wrong result buffer\n");
               printf ( "3rd) results at (%u,%u) are %f %f \n", i, j,
                              send_buffer[j][i], ref_out_data[j][i] );
            }
         }
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned j = 0; j < NUM_SRC; ++j ) {
         for (unsigned i = 0; i < sizeof(method_stack[j])/sizeof(method_stack[j][0]); ++i)
            if (method_stack[j][i] != NULL)
               yac_delete_interpolation_method(method_stack[j][i]);
         for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack[j]) /
              sizeof(interpolation_plan_element_stack[j][0]); ++i) {
            yac_free_interpolation_plan_element(interpolation_plan_element_stack[j][i]);
            free(interpolation_plan_element_stack[j][i]);
         }
         delete_interpolation_method_data_request_stack(data_request[j], 1);
      }

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_mask(&cell_mask_src[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_points(&cell_points_src[i]);

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
         yac_global_search_delete(search[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_domain(&l_domain_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_delete_grid(grid_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_delete_grid(grid_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_named_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_named_tgt[i]);

      yac_reset_interpolation_method();
#undef NUM_SRC
#undef NUM_TGT
   }
#endif
#ifdef TEST8
   {  // spmap recovery with three source and two target processes
      // checks request forwarding

      // the global source domain is a 3x2 grid:
      //
      //       08--13--09--14--10--15--11
      //       |       |       |       |
      //       07  03  09  04  11  05  12
      //       |       |       |       |
      //       04--06--05--08--06--10--07
      //       |       |       |       |
      //       01  00  03  01  05  02  06
      //       |       |       |       |
      //       00--00--01--02--02--04--03
      //
      // the global target domain is a 1x2 grid:
      //
      //        02--03--03
      //        |       |
      //        01  00  02
      //        |       |
      //        00--00--01
      //
      //---------------
      // setup
      //---------------

      // comm worlds on both processes

      #define NUM_SRC 3
      #define NUM_TGT 1

      struct communicator * comm_src[NUM_SRC], * comm_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_src[i] = yac_local_comm_new();

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_tgt[i] = yac_local_comm_new();

      // named comms for the domains on both processes
      struct communicator * comm_named_src[NUM_SRC], * comm_named_tgt[NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         comm_named_src[i] = yac_named_local_comm_new(comm_src[i], "src");

      for (unsigned i = 0; i < NUM_TGT; ++i)
         comm_named_tgt[i] = yac_named_local_comm_new(comm_tgt[i], "tgt");

      // define domains on source

      struct grid * grid_src[NUM_SRC];

      unsigned num_cells_src[NUM_SRC][2] = {{2,2}, {2,2}, {2,2}};

      unsigned cyclic_src[2] = {0,0};

      double coordinates_x_src[NUM_SRC][3] = {{0.0,1.0,2.0},
                                              {1.0,2.0,3.0},
                                              {1.0,2.0,3.0}};
      double coordinates_y_src[3] = {0.0,1.0,2.0};

      double coordinates_x_src_points[NUM_SRC][2] = {{0.5*YAC_RAD, 1.5*YAC_RAD},
                                                     {1.5*YAC_RAD, 2.5*YAC_RAD},
                                                     {1.5*YAC_RAD, 2.5*YAC_RAD}};

      double coordinates_y_src_points[2] = {0.5*YAC_RAD, 1.5*YAC_RAD};

      unsigned cell_mask_values_src[NUM_SRC][4] = {{1,1,1,1},
                                                   {1,1,1,0},
                                                   {1,0,1,1}};

      for ( unsigned j = 0; j < NUM_SRC; ++j )
         for ( unsigned i = 0; i < 3; ++i )
            coordinates_x_src[j][i] *= YAC_RAD;

      for ( unsigned i = 0; i < 3; ++i )
         coordinates_y_src[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_SRC; ++i)
         grid_src[i] =
            yac_reg2d_grid_new(coordinates_x_src[i], coordinates_y_src,
                               num_cells_src[i], cyclic_src);

      struct subdomain subdomain_src[NUM_SRC];

      int global_corner_ids_src[NUM_SRC][9] = {{0,1,2, 4,5,6, 8,9,10},
                                               {1,2,3, 5,6,7, 9,10,11},
                                               {1,2,3, 5,6,7, 9,10,11}};

      unsigned corner_is_on_rank_src[NUM_SRC][9] = {{-1,-1,2, -1,-1,1, -1,-1,1},
                                                    {0,2,2, 0,-1,-1, 0,-1,-1},
                                                    {0,-1,-1, 0,1,1, 0,1,1}};

      int global_cell_ids_src[NUM_SRC][9] = {{0,1,3,4},
                                             {1,2,4,5},
                                             {1,2,4,5}};

      unsigned cell_is_on_rank_src[NUM_SRC][9] = {{-1,2-1,1},
                                                  {0,2,0,-1},
                                                  {0,-1,0,1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_subdomain_unstruct(&subdomain_src[i], grid_src[i], comm_named_src[i],
                                    global_corner_ids_src[i], corner_is_on_rank_src[i],
                                    global_cell_ids_src[i], cell_is_on_rank_src[i],
                                    NULL, NULL);

      struct domain l_domain_src[NUM_SRC];

      struct subdomain * blocks_src[NUM_SRC] = {&subdomain_src[0],
                                                &subdomain_src[1],
                                                &subdomain_src[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_domain(
          &l_domain_src[i], comm_named_src[i], &blocks_src[i], 1, "domain_src");

      // define a domain on the target

      struct grid * grid_tgt[NUM_TGT];
      unsigned num_cells_tgt[2] = {1,1};
      unsigned cyclic_tgt[2] = {0,0};
      double coordinates_x_tgt[2] = {-0.5,0.5};
      double coordinates_y_tgt[2] = {0.5,1.5};

      double coordinates_x_tgt_point[1] = {0.0};
      double coordinates_y_tgt_point[1] = {1.0};

      for ( unsigned i = 0; i < 2; ++i ) coordinates_x_tgt[i] *= YAC_RAD;
      for ( unsigned i = 0; i < 2; ++i ) coordinates_y_tgt[i] *= YAC_RAD;

      for ( unsigned i = 0; i < 1; ++i ) coordinates_x_tgt_point[i] *= YAC_RAD;
      for ( unsigned i = 0; i < 1; ++i ) coordinates_y_tgt_point[i] *= YAC_RAD;

      for (unsigned i = 0; i < NUM_TGT; ++i)
        grid_tgt[i] =
          yac_reg2d_grid_new(coordinates_x_tgt, coordinates_y_tgt,
                             num_cells_tgt, cyclic_tgt);

      struct subdomain subdomain_tgt[NUM_TGT];

      int global_corner_ids_tgt[4] = {0,1,2,3};
      unsigned corner_is_on_rank_tgt[NUM_TGT][4] =
        {{-1,-1,-1,-1}};
      int global_cell_ids_tgt[1] = {0};
      unsigned cell_is_on_rank_tgt[NUM_TGT][1] = {{-1}};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_subdomain_unstruct(
          &subdomain_tgt[i], grid_tgt[i], comm_named_tgt[i],
          global_corner_ids_tgt, corner_is_on_rank_tgt[i],
          global_cell_ids_tgt, cell_is_on_rank_tgt[i], NULL, NULL);

      struct domain l_domain_tgt[NUM_TGT];

      struct subdomain * blocks_tgt[NUM_TGT] = {&subdomain_tgt[0]};

      for (unsigned i = 0; i < NUM_TGT; ++i)
        yac_init_domain(
          &l_domain_tgt[i], comm_named_tgt[i], &blocks_tgt[i], 1, "domain_tgt");

      // finish domain definition
      for (unsigned i = 0; i < NUM_SRC; ++i) yac_enddef_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i) yac_enddef_domain(&l_domain_tgt[i]);

      //-------------------------------------
      // init global search on both processes
      //-------------------------------------

      struct global_search * search[NUM_SRC + NUM_TGT];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        search[i] =
          yac_global_search_new(&l_domain_src[i], "domain_tgt", comm_src[i],
                                1,0, "component_p1", "component_p2");
      for (unsigned i = 0; i < NUM_TGT; ++i)
        search[NUM_SRC + i] =
          yac_global_search_new(&l_domain_tgt[i], "domain_src", comm_tgt[i],
                                0,1, "component_p2", "component_p1");

      //---------------------------------------
      // finish initialisation of global search
      //---------------------------------------

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
        yac_enddef_global_search(search[i]);

      //---------------------------------
      // initialise points on source side
      //---------------------------------

      struct points cell_points_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_init_points(&cell_points_src[i], grid_src[i], CELL,
                         coordinates_x_src_points[i], coordinates_y_src_points );

      //------------------------------------------
      // set up coupling point sets on source side
      //------------------------------------------

      struct points * points_per_subdomain[NUM_SRC] = {&cell_points_src[0],
                                                       &cell_points_src[1],
                                                       &cell_points_src[2]};
      struct points ** point_sets[NUM_SRC] = {&points_per_subdomain[0],
                                              &points_per_subdomain[1],
                                              &points_per_subdomain[2]};

      //-------------------------------
      // initialise mask on source side
      //-------------------------------

      struct mask cell_mask_src[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        yac_init_mask(&cell_mask_src[i], cell_mask_values_src[i],
                      &cell_points_src[i]);

      //--------------------------------
      // set up mask sets on source side
      //--------------------------------

      struct mask * masks_per_subdomain[NUM_SRC];
      struct mask ** mask_sets[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        masks_per_subdomain[i] = &cell_mask_src[i];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        mask_sets[i] = &masks_per_subdomain[i];

      //----------------------------------------
      // test generation of interpolation method
      //----------------------------------------

      struct interpolation_method * method_spmap[NUM_SRC];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        method_spmap[i] =
          yac_new_interpolation_method_spmap_mask(
            search[i], &l_domain_src[i], "spmap stack", &point_sets[i],
            &mask_sets[i], 1, 0, 1);

      //----------
      // do search
      //----------

      struct interpolation_method * method_stack[NUM_SRC][2] =
        {{method_spmap[0], NULL},
         {method_spmap[1], NULL},
         {method_spmap[2], NULL}};

      struct interpolation_plan_element
        * interpolation_plan_element_stack[NUM_SRC][2] =
          {{NULL, NULL},{NULL, NULL},{NULL, NULL}};
      int global_ids[NUM_SRC][1] = {{0},{-1},{-1}};
      double coordinates_xyz[NUM_SRC][1][3];
      unsigned num_points[NUM_SRC] = {1,0,0};
      unsigned num_interpolated_points[NUM_SRC][1];

      for (unsigned i = 0; i < NUM_SRC; ++i)
        LLtoXYZ(coordinates_x_tgt_point[0],
                coordinates_y_tgt_point[0],
                coordinates_xyz[i][0]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_do_search_interpolation_method(
            method_stack[i], interpolation_plan_element_stack[i], global_ids[i],
            coordinates_xyz[i], CELL, num_points[i], num_interpolated_points[i],
            1);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_enddef_interpolation_method(method_stack[i]);

      //---------------------
      // do the interpolation
      //---------------------

      double in_data_raw[NUM_SRC][2][4] = {{{2,5},{-1,-1,-1,-1}},
                                           {{-1},{-1,-1,-1,5}},
                                           {{-1},{-1,2,-1,-1}}};
      double * remote_data[NUM_SRC][1] = {{in_data_raw[0][0]},
                                          {in_data_raw[1][0]},
                                          {in_data_raw[2][0]}};
      double * subdomain_in_data[NUM_SRC][1] = {{in_data_raw[0][1]},
                                                {in_data_raw[1][1]},
                                                {in_data_raw[2][1]}};
      double ** in_data[NUM_SRC][2] = {{remote_data[0], subdomain_in_data[0]},
                                       {remote_data[1], subdomain_in_data[1]},
                                       {remote_data[2], subdomain_in_data[2]}};
      double send_buffer[NUM_SRC][2] = {{-1,-1},{-1,-1},{-1,-1}};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_execute_interpolation_plan_element(interpolation_plan_element_stack[i],
                                                in_data[i], send_buffer[i]);

      //---------------------------------
      // testing results of interpolation
      //---------------------------------

      struct interpolation_method_data_request data_request_[NUM_SRC];
      struct interpolation_method_data_request * data_request[NUM_SRC] =
         {&data_request_[0],&data_request_[1],&data_request_[2]};

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_get_source_exchange_data_interpolation_method(method_stack[i], &data_request[i], 1);

      int ref_requested_global_ids[2] = {2,5};
      int ref_requested_ranks[2] = {2,1};

      struct interpolation_method_data_request ref_data_request[NUM_SRC] =
         {{2,ref_requested_global_ids,ref_requested_ranks},
          {0,NULL,NULL}, {0,NULL,NULL}};

      unsigned ref_num_interpolated_points[NUM_SRC] = {1,0,0};
      int ref_global_ids[NUM_SRC][2] = {{0},{-1},{-1}};
      double ref_out_data[NUM_SRC][2] = {{7},{-1},{-1}};

      for (unsigned j = 0; j < NUM_SRC; ++j ) {

         check_data_request(data_request[j], &ref_data_request[j], 1);

         if (ref_num_interpolated_points[j] != num_interpolated_points[j][0])
            PUT_ERR( "3rd) wrong number of interpolated points\n");

         for (unsigned i = 0; i < ref_num_interpolated_points[j]; ++i) {
            if (ref_global_ids[j][i] != global_ids[j][i])
               PUT_ERR( "3rd) wrong global id order\n");
            // since the cells are on the sphere, we do not have perfect
            // rectangular cells and hence the reference data is very inaccurate...
            if ( fabs(send_buffer[j][i] - ref_out_data[j][i] ) > tol ) {
               PUT_ERR( "3rd) wrong result buffer\n");
               printf ( "3rd) results at (%u,%u) are %f %f \n", i, j,
                              send_buffer[j][i], ref_out_data[j][i] );
            }
         }
      }

      //---------------
      // cleanup
      //---------------

      for (unsigned j = 0; j < NUM_SRC; ++j ) {
         for (unsigned i = 0; i < sizeof(method_stack[j])/sizeof(method_stack[j][0]); ++i)
            if (method_stack[j][i] != NULL)
               yac_delete_interpolation_method(method_stack[j][i]);
         for (unsigned i = 0; i < sizeof(interpolation_plan_element_stack[j]) /
              sizeof(interpolation_plan_element_stack[j][0]); ++i) {
            yac_free_interpolation_plan_element(interpolation_plan_element_stack[j][i]);
            free(interpolation_plan_element_stack[j][i]);
         }
         delete_interpolation_method_data_request_stack(data_request[j], 1);
      }

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_mask(&cell_mask_src[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_points(&cell_points_src[i]);

      for (unsigned i = 0; i < NUM_SRC + NUM_TGT; ++i)
         yac_global_search_delete(search[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_domain(&l_domain_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_domain(&l_domain_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_delete_grid(grid_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_delete_grid(grid_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_tgt[i]);

      for (unsigned i = 0; i < NUM_SRC; ++i)
         yac_free_communicator(comm_named_src[i]);
      for (unsigned i = 0; i < NUM_TGT; ++i)
         yac_free_communicator(comm_named_tgt[i]);

      yac_reset_interpolation_method();
#undef NUM_SRC
#undef NUM_TGT
   }
#endif
   yac_comm_finalize();

   return TEST_EXIT_CODE;
}

