/**
 * @file test_mask.c
 *
 * @copyright Copyright  (C)  2013 DKRZ, MPI-M
 *
 * @version 1.0
 *
 * @author Moritz Hanke <hanke@dkrz.de>
 *
 */
/*
 * Keywords:
 * Maintainer: Moritz Hanke <hanke@dkrz.de>
 *             Rene Redler <rene.redler@mpimet.mpg.de>
 * URL: https://doc.redmine.dkrz.de/YAC/html/index.html
 *
 * This file is part of YAC.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "tests.h"
#include "grid.h"
#include "grid_reg2d.h"
#include "points.h"
#include "mask.h"

int main (void) {

   unsigned num_cells[2] = {2,2};
   unsigned cyclic[2] = {0,0};
   struct grid * grid[2];
   struct points points_corner, points_cell;

   { // set up data structures required for the testing the mask

      grid[0] = yac_reg2d_grid_new(NULL, NULL, num_cells, cyclic);
      yac_init_points(&points_corner, grid[0], CORNER, NULL, NULL);

      grid[1] = yac_reg2d_grid_new(NULL, NULL, num_cells, cyclic);
      yac_init_points(&points_cell, grid[1], CELL, NULL, NULL);
   }

   {
      struct mask mask;

      unsigned mask_values[] = {0,1,0,1};

      yac_init_mask(&mask, mask_values, &points_cell);

      yac_free_mask(&mask);
   }

   {
      struct mask mask;

      unsigned mask_values[] = {0,0,0,1,1,1,0,0,0};

      yac_init_mask(&mask, mask_values, &points_corner);

      yac_free_mask(&mask);
   }

   // clean up
   yac_free_points(&points_cell);
   yac_free_points(&points_corner);
   yac_delete_grid(grid[0]);
   yac_delete_grid(grid[1]);

   return TEST_EXIT_CODE;
}

